@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaldem.cpp 24723 2012-07-31 22:20:20Z rouault $
+ * $Id: gdaldem.cpp 27613 2014-08-30 15:55:30Z rouault $
  *
  * Project:  GDAL DEM Utilities
  * Purpose:  
@@ -10,7 +10,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2006, 2009 Matthew Perry 
- * Copyright (c) 2009 Even Rouault
+ * Copyright (c) 2009-2013, Even Rouault <even dot rouault at mines-paris dot org>
  * Portions derived from GRASS 4.1 (public domain) See 
  * http://trac.osgeo.org/gdal/ticket/2975 for more information regarding 
  * history of this code
@@ -91,7 +91,7 @@
 #include "gdal_priv.h"
 #include "commonutils.h"
 
-CPL_CVSID("$Id: gdaldem.cpp 24723 2012-07-31 22:20:20Z rouault $");
+CPL_CVSID("$Id: gdaldem.cpp 27613 2014-08-30 15:55:30Z rouault $");
 
 #ifndef M_PI
 # define M_PI  3.1415926535897932384626433832795
@@ -103,7 +103,7 @@ CPL_CVSID("$Id: gdaldem.cpp 24723 2012-07-31 22:20:20Z rouault $");
 /*                               Usage()                                */
 /************************************************************************/
 
-static void Usage()
+static void Usage(const char* pszErrorMsg = NULL)
 
 {
     printf( " Usage: \n"
@@ -111,7 +111,7 @@ static void Usage()
             "     gdaldem hillshade input_dem output_hillshade \n"
             "                 [-z ZFactor (default=1)] [-s scale* (default=1)] \n"
             "                 [-az Azimuth (default=315)] [-alt Altitude (default=45)]\n"
-            "                 [-alg ZevenbergenThorne]\n"
+            "                 [-alg ZevenbergenThorne] [-combined]\n"
             "                 [-compute_edges] [-b Band (default=1)] [-of format] [-co \"NAME=VALUE\"]* [-q]\n"
             "\n"
             " - To generates a slope map from any GDAL-supported elevation raster :\n\n"
@@ -148,6 +148,10 @@ static void Usage()
             " Notes : \n"
             "   Scale is the ratio of vertical units to horizontal\n"
             "    for Feet:Latlong use scale=370400, for Meters:LatLong use scale=111120 \n\n");
+
+    if( pszErrorMsg != NULL )
+        fprintf(stderr, "\nFAILURE: %s\n", pszErrorMsg);
+
     exit( 1 );
 }
 
@@ -442,6 +446,7 @@ typedef struct
     double cos_altRadians_mul_z_scale_factor;
     double azRadians;
     double square_z_scale_factor;
+    double square_M_PI_2;
 } GDALHillshadeAlgData;
 
 /* Unoptimized formulas are :
@@ -493,6 +498,41 @@ float GDALHillshadeAlg (float* afWin, float fDstNoDataValue, void* pData)
     return (float) cang;
 }
 
+float GDALHillshadeCombinedAlg (float* afWin, float fDstNoDataValue, void* pData)
+{
+    GDALHillshadeAlgData* psData = (GDALHillshadeAlgData*)pData;
+    double x, y, aspect, xx_plus_yy, cang;
+    
+    // First Slope ...
+    x = ((afWin[0] + afWin[3] + afWin[3] + afWin[6]) -
+        (afWin[2] + afWin[5] + afWin[5] + afWin[8])) / psData->ewres;
+
+    y = ((afWin[6] + afWin[7] + afWin[7] + afWin[8]) -
+        (afWin[0] + afWin[1] + afWin[1] + afWin[2])) / psData->nsres;
+
+    xx_plus_yy = x * x + y * y;
+
+    // ... then aspect...
+    aspect = atan2(y,x);
+    double slope = xx_plus_yy * psData->square_z_scale_factor;
+
+    // ... then the shade value
+    cang = acos((psData->sin_altRadians -
+           psData->cos_altRadians_mul_z_scale_factor * sqrt(xx_plus_yy) *
+           sin(aspect - psData->azRadians)) /
+           sqrt(1 + slope));
+
+    // combined shading
+    cang = 1 - cang * atan(sqrt(slope)) / psData->square_M_PI_2;
+
+    if (cang <= 0.0) 
+        cang = 1.0;
+    else
+        cang = 1.0 + (254.0 * cang);
+        
+    return (float) cang;
+}
+
 float GDALHillshadeZevenbergenThorneAlg (float* afWin, float fDstNoDataValue, void* pData)
 {
     GDALHillshadeAlgData* psData = (GDALHillshadeAlgData*)pData;
@@ -522,6 +562,39 @@ float GDALHillshadeZevenbergenThorneAlg (float* afWin, float fDstNoDataValue, vo
     return (float) cang;
 }
 
+float GDALHillshadeZevenbergenThorneCombinedAlg (float* afWin, float fDstNoDataValue, void* pData)
+{
+    GDALHillshadeAlgData* psData = (GDALHillshadeAlgData*)pData;
+    double x, y, aspect, xx_plus_yy, cang;
+    
+    // First Slope ...
+    x = (afWin[3] - afWin[5]) / psData->ewres;
+
+    y = (afWin[7] - afWin[1]) / psData->nsres;
+
+    xx_plus_yy = x * x + y * y;
+
+    // ... then aspect...
+    aspect = atan2(y,x);
+    double slope = xx_plus_yy * psData->square_z_scale_factor;
+
+    // ... then the shade value
+    cang = acos((psData->sin_altRadians -
+           psData->cos_altRadians_mul_z_scale_factor * sqrt(xx_plus_yy) *
+           sin(aspect - psData->azRadians)) /
+           sqrt(1 + slope));
+
+    // combined shading
+    cang = 1 - cang * atan(sqrt(slope)) / psData->square_M_PI_2;
+
+    if (cang <= 0.0) 
+        cang = 1.0;
+    else
+        cang = 1.0 + (254.0 * cang);
+        
+    return (float) cang;
+}
+
 void*  GDALCreateHillshadeData(double* adfGeoTransform,
                                double z,
                                double scale,
@@ -541,6 +614,7 @@ void*  GDALCreateHillshadeData(double* adfGeoTransform,
     pData->cos_altRadians_mul_z_scale_factor =
         cos(alt * degreesToRadians) * z_scale_factor;
     pData->square_z_scale_factor = z_scale_factor * z_scale_factor;
+    pData->square_M_PI_2 = (M_PI*M_PI)/4;
     return pData;
 }
 
@@ -2109,6 +2183,15 @@ double GDALGeneric3x3RasterBand::GetNoDataValue( int* pbHasNoData )
     return poGDS->dfDstNoDataValue;
 }
 
+/************************************************************************/
+/*                            ArgIsNumeric()                            */
+/************************************************************************/
+
+static int ArgIsNumeric( const char *pszArg )
+
+{
+    return CPLGetValueType(pszArg) != CPL_VALUE_STRING;
+}
 /************************************************************************/
 /*                                main()                                */
 /************************************************************************/
@@ -2124,6 +2207,10 @@ typedef enum
     ROUGHNESS
 } Algorithm;
 
+#define CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(nExtraArg) \
+    do { if (i + nExtraArg >= argc) \
+        Usage(CPLSPrintf("%s option requires %d argument(s)", argv[i], nExtraArg)); } while(0)
+
 int main( int argc, char ** argv )
 
 {
@@ -2162,7 +2249,7 @@ int main( int argc, char ** argv )
     
     int bComputeAtEdges = FALSE;
     int bZevenbergenThorne = FALSE;
-
+    int bCombined = FALSE;
     int bQuiet = FALSE;
     
     /* Check strict compilation and runtime library version as we use C++ API */
@@ -2172,8 +2259,7 @@ int main( int argc, char ** argv )
     argc = GDALGeneralCmdLineProcessor( argc, &argv, 0 );
     if( argc < 2 )
     {
-        fprintf(stderr, "Not enough arguments\n");
-        Usage();
+        Usage("Not enough arguments.");
     }
 
     if( EQUAL(argv[1], "--utility_version") || EQUAL(argv[1], "--utility-version") )
@@ -2182,6 +2268,8 @@ int main( int argc, char ** argv )
                 argv[0], GDAL_RELEASE_NAME, GDALVersionInfo("RELEASE_NAME"));
         return 0;
     }
+    else if( EQUAL(argv[1],"--help") )
+        Usage();
     else if ( EQUAL(argv[1], "shade") || EQUAL(argv[1], "hillshade") )
     {
         eUtilityMode = HILL_SHADE;
@@ -2212,8 +2300,7 @@ int main( int argc, char ** argv )
     }
     else
     {
-        fprintf(stderr, "Missing valid sub-utility mention\n");
-        Usage();
+        Usage("Missing valid sub-utility mention.");
     }
 
 /* -------------------------------------------------------------------- */
@@ -2221,25 +2308,29 @@ int main( int argc, char ** argv )
 /* -------------------------------------------------------------------- */
     for(int i = 2; i < argc; i++ )
     {
-        if( eUtilityMode == HILL_SHADE && i + 1 < argc &&
+        if( eUtilityMode == HILL_SHADE &&
             (EQUAL(argv[i], "--z") || EQUAL(argv[i], "-z")))
         {
-            z = atof(argv[++i]);
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
+            ++i;
+            if( !ArgIsNumeric(argv[i]) )
+                Usage();
+            z = atof(argv[i]);
         }
         else if ( eUtilityMode == SLOPE && EQUAL(argv[i], "-p"))
         {
             slopeFormat = 0;
         }
         else if ( (eUtilityMode == HILL_SHADE || eUtilityMode == SLOPE ||
-                   eUtilityMode == ASPECT) && EQUAL(argv[i], "-alg") && i + 1 < argc)
+                   eUtilityMode == ASPECT) && EQUAL(argv[i], "-alg") )
         {
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
             i ++;
             if (EQUAL(argv[i], "ZevenbergenThorne"))
                 bZevenbergenThorne = TRUE;
             else if (!EQUAL(argv[i], "Horn"))
             {
-                fprintf(stderr, "Wrong value for alg : %s\n", argv[i]);
-                Usage();
+                Usage(CPLSPrintf("Wrong value for alg : %s.", argv[i]));
             }
         }
         else if ( eUtilityMode == ASPECT && EQUAL(argv[i], "-trigonometric"))
@@ -2258,32 +2349,51 @@ int main( int argc, char ** argv )
         {
             eColorSelectionMode = COLOR_SELECTION_NEAREST_ENTRY;
         }
-        else if( i + 1 < argc &&
+        else if( 
             (EQUAL(argv[i], "--s") || 
              EQUAL(argv[i], "-s") ||
              EQUAL(argv[i], "--scale") ||
              EQUAL(argv[i], "-scale"))
           )
         {
-            scale = atof(argv[++i]);
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
+            ++i;
+            if( !ArgIsNumeric(argv[i]) )
+                Usage();
+            scale = atof(argv[i]);
         }
-        else if( eUtilityMode == HILL_SHADE && i + 1 < argc &&
+        else if( eUtilityMode == HILL_SHADE &&
             (EQUAL(argv[i], "--az") || 
              EQUAL(argv[i], "-az") ||
              EQUAL(argv[i], "--azimuth") ||
              EQUAL(argv[i], "-azimuth"))
           )
         {
-            az = atof(argv[++i]);
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
+            ++i;
+            if( !ArgIsNumeric(argv[i]) )
+                Usage();
+            az = atof(argv[i]);
         }
-        else if( eUtilityMode == HILL_SHADE && i + 1 < argc &&
+        else if( eUtilityMode == HILL_SHADE &&
             (EQUAL(argv[i], "--alt") || 
              EQUAL(argv[i], "-alt") ||
              EQUAL(argv[i], "--alt") ||
              EQUAL(argv[i], "-alt"))
           )
         {
-            alt = atof(argv[++i]);
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
+            ++i;
+            if( !ArgIsNumeric(argv[i]) )
+                Usage();
+            alt = atof(argv[i]);
+        }
+        else if( eUtilityMode == HILL_SHADE &&
+            (EQUAL(argv[i], "-combined") || 
+             EQUAL(argv[i], "--combined"))
+          )
+        {
+            bCombined = TRUE;
         }
         else if( eUtilityMode == COLOR_RELIEF &&
                  EQUAL(argv[i], "-alpha"))
@@ -2307,20 +2417,20 @@ int main( int argc, char ** argv )
             pfnProgress = GDALDummyProgress;
             bQuiet = TRUE;
         }
-        else if( EQUAL(argv[i],"-co") && i < argc-1 )
+        else if( EQUAL(argv[i],"-co") )
         {
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
             papszCreateOptions = CSLAddString( papszCreateOptions, argv[++i] );
         }
-        else if( EQUAL(argv[i],"-of") && i < argc-1 )
+        else if( EQUAL(argv[i],"-of") )
         {
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
             pszFormat = argv[++i];
             bFormatExplicitelySet = TRUE;
         }
         else if( argv[i][0] == '-' )
         {
-            fprintf( stderr, "Option %s incomplete, or not recognised.\n\n", 
-                    argv[i] );
-            Usage();
+            Usage(CPLSPrintf("Unknown option name '%s'", argv[i]));
         }
         else if( pszSrcFilename == NULL )
         {
@@ -2335,23 +2445,20 @@ int main( int argc, char ** argv )
             pszDstFilename = argv[i];
         }
         else
-            Usage();
+            Usage("Too many command options.");
     }
 
     if( pszSrcFilename == NULL )
     {
-        fprintf( stderr, "Missing source.\n\n" );
-        Usage();
+        Usage("Missing source.");
     }
     if ( eUtilityMode == COLOR_RELIEF && pszColorFilename == NULL )
     {
-        fprintf( stderr, "Missing color file.\n\n" );
-        Usage();
+        Usage("Missing color file.");
     }
     if( pszDstFilename == NULL )
     {
-        fprintf( stderr, "Missing destination.\n\n" );
-        Usage();
+        Usage("Missing destination.");
     }
 
     GDALAllRegister();
@@ -2429,9 +2536,19 @@ int main( int argc, char ** argv )
                                            az,
                                            bZevenbergenThorne);
         if (bZevenbergenThorne)
-            pfnAlg = GDALHillshadeZevenbergenThorneAlg;
+        {
+            if(!bCombined)
+                pfnAlg = GDALHillshadeZevenbergenThorneAlg;
+            else
+                pfnAlg = GDALHillshadeZevenbergenThorneCombinedAlg;
+        }
         else
-            pfnAlg = GDALHillshadeAlg;
+        {
+            if(!bCombined)
+                pfnAlg = GDALHillshadeAlg;
+            else
+                pfnAlg = GDALHillshadeCombinedAlg;
+        }
     }
     else if (eUtilityMode == SLOPE)
     {
