@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalenhance.cpp 17553 2009-08-21 14:27:19Z rouault $
+ * $Id: gdalenhance.cpp 27044 2014-03-16 23:41:27Z rouault $
  *
  * Project:  GDAL Utilities
  * Purpose:  Commandline application to do image enhancement. 
@@ -7,6 +7,7 @@
  *
  * ****************************************************************************
  * Copyright (c) 2007, Frank Warmerdam <warmerdam@pobox.com>
+ * Copyright (c) 2007-2011, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,8 +33,9 @@
 #include "cpl_conv.h"
 #include "cpl_multiproc.h"
 #include "vrt/vrtdataset.h"
+#include "commonutils.h"
 
-CPL_CVSID("$Id: gdalenhance.cpp 17553 2009-08-21 14:27:19Z rouault $");
+CPL_CVSID("$Id: gdalenhance.cpp 27044 2014-03-16 23:41:27Z rouault $");
 
 static int
 ComputeEqualizationLUTs( GDALDatasetH hDataset,  int nLUTBins,
@@ -87,6 +89,7 @@ int main( int argc, char ** argv )
     GDALDatasetH	hDataset, hOutDS;
     int			i;
     const char		*pszSource=NULL, *pszDest=NULL, *pszFormat = "GTiff";
+    int bFormatExplicitelySet = FALSE;
     GDALDriverH		hDriver;
     GDALDataType	eOutputType = GDT_Unknown;
     char                **papszCreateOptions = NULL;
@@ -99,6 +102,7 @@ int main( int argc, char ** argv )
     int               **papanLUTs = NULL;
     int                 iBand;
     const char         *pszConfigFile = NULL;
+    int                 bQuiet = FALSE;
 
     /* Check strict compilation and runtime library version as we use C++ API */
     if (! GDAL_CHECK_VERSION(argv[0]))
@@ -124,7 +128,10 @@ int main( int argc, char ** argv )
             return 0;
         }
         else if( EQUAL(argv[i],"-of") && i < argc-1 )
+        {
             pszFormat = argv[++i];
+            bFormatExplicitelySet = TRUE;
+        }
 
         else if( EQUAL(argv[i],"-ot") && i < argc-1 )
         {
@@ -144,8 +151,6 @@ int main( int argc, char ** argv )
             {
                 printf( "Unknown output pixel type: %s\n", argv[i+1] );
                 Usage();
-                GDALDestroyDriverManager();
-                exit( 2 );
             }
             i++;
         }
@@ -186,6 +191,7 @@ int main( int argc, char ** argv )
         else if( EQUAL(argv[i],"-quiet") )
         {
             pfnProgress = GDALDummyProgress;
+            bQuiet = TRUE;
         }
 
         else if( argv[i][0] == '-' )
@@ -193,8 +199,6 @@ int main( int argc, char ** argv )
             printf( "Option %s incomplete, or not recognised.\n\n", 
                     argv[i] );
             Usage();
-            GDALDestroyDriverManager();
-            exit( 2 );
         }
         else if( pszSource == NULL )
         {
@@ -209,16 +213,12 @@ int main( int argc, char ** argv )
         {
             printf( "Too many command options.\n\n" );
             Usage();
-            GDALDestroyDriverManager();
-            exit( 2 );
         }
     }
 
     if( pszSource == NULL )
     {
         Usage();
-        GDALDestroyDriverManager();
-        exit( 10 );
     }
 
 /* -------------------------------------------------------------------- */
@@ -263,14 +263,11 @@ int main( int argc, char ** argv )
         }
         printf( "\n" );
         Usage();
-        
-        GDALClose( hDataset );
-        GDALDestroyDriverManager();
-        CSLDestroy( argv );
-        CSLDestroy( papszCreateOptions );
-        exit( 1 );
     }
 
+    if (!bQuiet && pszDest != NULL && !bFormatExplicitelySet)
+        CheckExtensionConsistency(pszDest, pszFormat);
+
 /* -------------------------------------------------------------------- */
 /*      If histogram equalization is requested, do it now.              */
 /* -------------------------------------------------------------------- */
@@ -623,9 +620,9 @@ static CPLErr EnhancerCallback( void *hCBData,
         iBin = MAX(0,MIN(psEInfo->nLUTBins-1,iBin));
 
         if( psEInfo->panLUT )
-            pabyOutImage[iPixel] = psEInfo->panLUT[iBin];
+            pabyOutImage[iPixel] = (GByte) psEInfo->panLUT[iBin];
         else
-            pabyOutImage[iPixel] = iBin;
+            pabyOutImage[iPixel] = (GByte) iBin;
     }
 
     CPLFree( pafSrcImage );
