@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: argdataset.cpp 24855 2012-08-25 21:12:55Z rouault $
+ * $Id: argdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
  *
  * Project:  Azavea Raster Grid format driver.
  * Purpose:  Implements support for reading and writing Azavea Raster Grid
@@ -8,6 +8,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2012, David Zwarg <dzwarg@azavea.com>
+ * Copyright (c) 2012-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,11 +31,10 @@
 
 #include "rawdataset.h"
 #include "cpl_string.h"
-#include "jsonc/json.h"
-#include "jsonc/json_util.h"
+#include <json.h>
 #include <ogr_spatialref.h>
 
-CPL_CVSID("$Id: argdataset.cpp 24855 2012-08-25 21:12:55Z rouault $");
+CPL_CVSID("$Id: argdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
 
 #define MAX_FILENAME_LEN 4096
 
@@ -136,9 +136,9 @@ CPLString GetJsonFilename(CPLString pszFilename)
 json_object * GetJsonObject(CPLString pszFilename) 
 {
     json_object * pJSONObject = NULL;
-    CPLString pszJSONFilename = GetJsonFilename(pszFilename);
+    CPLString osJSONFilename = GetJsonFilename(pszFilename);
 
-    pJSONObject = json_object_from_file((char *)pszJSONFilename.c_str());
+    pJSONObject = json_object_from_file((char *)osJSONFilename.c_str());
     if (pJSONObject == (struct json_object*)error_ptr(-1) || pJSONObject == NULL) {
         CPLDebug("ARGDataset", "GetJsonObject(): "
             "Could not parse JSON file.");
@@ -204,9 +204,9 @@ int GetJsonValueInt(json_object * pJSONObject, CPLString pszKey)
 char ** ARGDataset::GetFileList()
 {
     char **papszFileList = GDALPamDataset::GetFileList();
-    CPLString pszJSONFilename = GetJsonFilename(pszFilename);
+    CPLString osJSONFilename = GetJsonFilename(pszFilename);
 
-    papszFileList = CSLAddString( papszFileList, pszJSONFilename );
+    papszFileList = CSLAddString( papszFileList, osJSONFilename );
 
     return papszFileList;
 }
@@ -577,15 +577,18 @@ GDALDataset *ARGDataset::Open( GDALOpenInfo * poOpenInfo )
 /*                          CreateCopy()                                */
 /************************************************************************/
 GDALDataset * ARGDataset::CreateCopy( const char * pszFilename, 
-    GDALDataset * poSrcDS, int bStrict, char ** papszOptions, 
-    GDALProgressFunc pfnProgress, void * pProgressData ) 
+                                      GDALDataset * poSrcDS,
+                                      CPL_UNUSED int bStrict,
+                                      CPL_UNUSED char ** papszOptions, 
+                                      CPL_UNUSED GDALProgressFunc pfnProgress,
+                                      CPL_UNUSED void * pProgressData ) 
 {
     int nBands = poSrcDS->GetRasterCount();
     int nXSize = poSrcDS->GetRasterXSize();
     int nYSize = poSrcDS->GetRasterYSize();
     int nXBlockSize, nYBlockSize, nPixelOffset;
     GDALDataType eType;
-    CPLString pszJSONFilename;
+    CPLString osJSONFilename;
     CPLString pszDataType;
     json_object * poJSONObject = NULL;
     double adfTransform[6];
@@ -674,12 +677,7 @@ GDALDataset * ARGDataset::CreateCopy( const char * pszFilename,
     /********************************************************************/
     /* Create JSON companion file.                                      */
     /********************************************************************/
-    pszJSONFilename = GetJsonFilename(pszFilename);
-    if (pszJSONFilename == NULL) {
-        CPLError( CE_Failure, CPLE_NotSupported, 
-                  "ARG driver can't determine filename for companion file.");
-        return NULL;
-    }
+    osJSONFilename = GetJsonFilename(pszFilename);
 
     poJSONObject = json_object_new_object();
 
@@ -689,7 +687,7 @@ GDALDataset * ARGDataset::CreateCopy( const char * pszFilename,
     if ( pszLayer == NULL) {
         // Set the layer
         json_object_object_add(poJSONObject, "layer", json_object_new_string(
-            CPLGetBasename(pszJSONFilename)
+            CPLGetBasename(osJSONFilename)
         ));
     }
     else {
@@ -728,7 +726,7 @@ GDALDataset * ARGDataset::CreateCopy( const char * pszFilename,
         json_object_object_add(poJSONObject, "epsg", json_object_new_int(nSrs));
     }
 
-    if (json_object_to_file((char *)pszJSONFilename.c_str(), poJSONObject) < 0) {
+    if (json_object_to_file((char *)osJSONFilename.c_str(), poJSONObject) < 0) {
         CPLError( CE_Failure, CPLE_NotSupported, 
                   "ARG driver can't write companion file.");
 
@@ -748,7 +746,7 @@ GDALDataset * ARGDataset::CreateCopy( const char * pszFilename,
               "ARG driver can't create data file %s.", pszFilename);
 
         // remove JSON file
-        VSIUnlink( pszJSONFilename.c_str() );
+        VSIUnlink( osJSONFilename.c_str() );
 
         return NULL;
     }
