@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ersdataset.cpp 23029 2011-09-03 10:15:51Z rouault $
+ * $Id: ersdataset.cpp 27433 2014-06-04 19:21:16Z rouault $
  *
  * Project:  ERMapper .ers Driver
  * Purpose:  Implementation of .ers driver.
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2007, Frank Warmerdam <warmerdam@pobox.com>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +33,7 @@
 #include "cpl_string.h"
 #include "ershdrnode.h"
 
-CPL_CVSID("$Id: ersdataset.cpp 23029 2011-09-03 10:15:51Z rouault $");
+CPL_CVSID("$Id: ersdataset.cpp 27433 2014-06-04 19:21:16Z rouault $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -98,6 +99,7 @@ class ERSDataset : public RawDataset
     virtual CPLErr SetGCPs( int nGCPCount, const GDAL_GCP *pasGCPList,
                             const char *pszGCPProjection );
 
+    virtual char      **GetMetadataDomainList();
     virtual const char *GetMetadataItem( const char * pszName,
                                      const char * pszDomain = "" );
     virtual char      **GetMetadata( const char * pszDomain = "" );
@@ -218,6 +220,17 @@ void ERSDataset::FlushCache()
     RawDataset::FlushCache();
 }
 
+/************************************************************************/
+/*                      GetMetadataDomainList()                         */
+/************************************************************************/
+
+char **ERSDataset::GetMetadataDomainList()
+{
+    return BuildMetadataDomainList(GDALPamDataset::GetMetadataDomainList(),
+                                   TRUE,
+                                   "ERS", NULL);
+}
+
 /************************************************************************/
 /*                           GetMetadataItem()                          */
 /************************************************************************/
@@ -540,7 +553,15 @@ CPLErr ERSDataset::SetGeoTransform( double *padfTransform )
                    CPLString().Printf( "%.15g", adfGeoTransform[0] ) );
     poHeader->Set( "RasterInfo.RegistrationCoord.Northings", 
                    CPLString().Printf( "%.15g", adfGeoTransform[3] ) );
-    
+
+    if( CPLAtof(poHeader->Find("RasterInfo.RegistrationCellX", "0")) != 0.0 ||
+        CPLAtof(poHeader->Find("RasterInfo.RegistrationCellY", "0")) != 0.0 ) 
+    {
+        // Reset RegistrationCellX/Y to 0 if the header gets rewritten (#5493)
+        poHeader->Set("RasterInfo.RegistrationCellX", "0");
+        poHeader->Set("RasterInfo.RegistrationCellY", "0");
+    }
+
     return CE_None;
 }
 
@@ -687,9 +708,9 @@ void ERSDataset::ReadGCPs()
     osDatum = poHeader->Find( "RasterInfo.WarpControl.CoordinateSpace.Datum", "" );
     osUnits = poHeader->Find( "RasterInfo.WarpControl.CoordinateSpace.Units", "" );
 
-    oSRS.importFromERM( osProj.size() ? osProj : "RAW",
-                        osDatum.size() ? osDatum : "WGS84",
-                        osUnits.size() ? osUnits : "METERS" );
+    oSRS.importFromERM( osProj.size() ? osProj.c_str() : "RAW",
+                        osDatum.size() ? osDatum.c_str() : "WGS84",
+                        osUnits.size() ? osUnits.c_str() : "METERS" );
 
     CPLFree( pszGCPProjection );
     oSRS.exportToWkt( &pszGCPProjection );
@@ -1045,9 +1066,9 @@ GDALDataset *ERSDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->osDatum = poHeader->Find( "CoordinateSpace.Datum", "" );
     poDS->osUnits = poHeader->Find( "CoordinateSpace.Units", "" );
 
-    oSRS.importFromERM( poDS->osProj.size() ? poDS->osProj : "RAW",
-                        poDS->osDatum.size() ? poDS->osDatum : "WGS84",
-                        poDS->osUnits.size() ? poDS->osUnits : "METERS" );
+    oSRS.importFromERM( poDS->osProj.size() ? poDS->osProj.c_str() : "RAW",
+                        poDS->osDatum.size() ? poDS->osDatum.c_str() : "WGS84",
+                        poDS->osUnits.size() ? poDS->osUnits.c_str() : "METERS" );
 
     CPLFree( poDS->pszProjection );
     oSRS.exportToWkt( &(poDS->pszProjection) );
@@ -1088,17 +1109,28 @@ GDALDataset *ERSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Adjust if we have a registration cell.                          */
 /* -------------------------------------------------------------------- */
-    int iCellX = atoi(poHeader->Find("RasterInfo.RegistrationCellX", "1"));
-    int iCellY = atoi(poHeader->Find("RasterInfo.RegistrationCellY", "1"));
+
+    /* http://geospatial.intergraph.com/Libraries/Tech_Docs/ERDAS_ER_Mapper_Customization_Guide.sflb.ashx */
+    /* Page 27 : */
+    /* RegistrationCellX and RegistrationCellY : The image X and Y
+       coordinates of the cell which corresponds to the Registration
+       Coordinate. Note that the RegistrationCellX and
+       RegistrationCellY can be fractional values. If
+       RegistrationCellX and RegistrationCellY are not specified,
+       they are assumed to be (0,0), which is the top left corner of the
+       image.
+       */
+    double dfCellX = CPLAtof(poHeader->Find("RasterInfo.RegistrationCellX", "0"));
+    double dfCellY = CPLAtof(poHeader->Find("RasterInfo.RegistrationCellY", "0"));
 
     if( poDS->bGotTransform )
     {
         poDS->adfGeoTransform[0] -=
-            (iCellX-1) * poDS->adfGeoTransform[1]
-            + (iCellY-1) * poDS->adfGeoTransform[2];
+            dfCellX * poDS->adfGeoTransform[1]
+            + dfCellY * poDS->adfGeoTransform[2];
         poDS->adfGeoTransform[3] -= 
-            (iCellX-1) * poDS->adfGeoTransform[4]
-            + (iCellY-1) * poDS->adfGeoTransform[5];
+            dfCellX * poDS->adfGeoTransform[4]
+            + dfCellY * poDS->adfGeoTransform[5];
     }
 
 /* -------------------------------------------------------------------- */
