@@ -31,6 +31,8 @@
 //#include "mymapf.h"
 #include "clock.h"
 
+#include "cpl_port.h"
+
 #define GRIB_UNSIGN_INT3(a,b,c) ((a<<16)+(b<<8)+c)
 
 /*****************************************************************************
@@ -815,7 +817,7 @@ int ReadGrib2Record (DataSource &fp, sChar f_unit, double **Grib_Data,
                      uInt4 *grib_DataLen, grib_MetaData *meta,
                      IS_dataType *IS, int subgNum, double majEarth,
                      double minEarth, int simpVer, sInt4 *f_endMsg,
-                     LatLon *lwlf, LatLon *uprt)
+                     CPL_UNUSED LatLon *lwlf, CPL_UNUSED LatLon *uprt)
 {
    sInt4 l3264b;        /* Number of bits in a sInt4.  Needed by FORTRAN
                          * unpack library to determine if system has a 4
