@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: northwood.cpp 24120 2012-03-15 19:41:49Z warmerdam $
+ * $Id: northwood.cpp 27729 2014-09-24 00:40:16Z goatbar $
  *
  * Project:  GRC/GRD Reader
  * Purpose:  Northwood Format basic implementation
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2007, Waypoint Information Technology
+ * Copyright (c) 2009-2011, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -479,9 +480,8 @@ void nwtCloseGrid( NWT_GRID * pGrd )
         return;
 }
 
-void nwtGetRow( NWT_GRID * pGrd )
+void nwtGetRow( CPL_UNUSED NWT_GRID * pGrd )
 {
-
 }
 
 void nwtPrintGridHeader( NWT_GRID * pGrd )
