@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: sagadataset.cpp 22699 2011-07-11 18:12:32Z rouault $
+ * $Id: sagadataset.cpp 27044 2014-03-16 23:41:27Z rouault $
  * Project:  SAGA GIS Binary Driver
  * Purpose:  Implements the SAGA GIS Binary Grid Format.
  * Author:   Volker Wichmann, wichmann@laserdata.at
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2009, Volker Wichmann <wichmann@laserdata.at>
+ * Copyright (c) 2009-2011, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,8 +35,9 @@
 #include <assert.h>
 
 #include "gdal_pam.h"
+#include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: sagadataset.cpp 22699 2011-07-11 18:12:32Z rouault $");
+CPL_CVSID("$Id: sagadataset.cpp 27044 2014-03-16 23:41:27Z rouault $");
 
 #ifndef INT_MAX
 # define INT_MAX 2147483647
@@ -75,8 +77,10 @@ class SAGADataset : public GDALPamDataset
 									double dfCellsize, double dfNoData,
 									double dfZFactor, bool bTopToBottom );
     VSILFILE				*fp;
+    char                *pszProjection;
 
   public:
+        SAGADataset();
 		~SAGADataset();
 
 		static GDALDataset		*Open( GDALOpenInfo * );
@@ -90,6 +94,8 @@ class SAGADataset : public GDALPamDataset
 										GDALProgressFunc pfnProgress,
 										void *pProgressData );
 		
+        virtual const char     *GetProjectionRef(void);
+        virtual CPLErr          SetProjection( const char * );
         virtual char          **GetFileList();
 
 		CPLErr					GetGeoTransform( double *padfGeoTransform );
@@ -299,16 +305,23 @@ double SAGARasterBand::GetNoDataValue( int * pbSuccess )
     return m_NoData;
 }
 
-
 /************************************************************************/
 /* ==================================================================== */
 /*                              SAGADataset	                            */
 /* ==================================================================== */
 /************************************************************************/
 
+
+SAGADataset::SAGADataset()
+
+{
+    pszProjection = CPLStrdup("");
+}
+
 SAGADataset::~SAGADataset()
 
 {
+    CPLFree( pszProjection );
     FlushCache();
     if( fp != NULL )
         VSIFCloseL( fp );
@@ -323,7 +336,7 @@ char** SAGADataset::GetFileList()
 {
     CPLString osPath = CPLGetPath( GetDescription() );
     CPLString osName = CPLGetBasename( GetDescription() );
-    CPLString osHDRFilename;
+    VSIStatBufL sStatBuf;
 
     char **papszFileList = NULL;
 
@@ -331,12 +344,74 @@ char** SAGADataset::GetFileList()
     papszFileList = GDALPamDataset::GetFileList();
 
     // Header file.
-    osHDRFilename = CPLFormCIFilename( osPath, osName, ".sgrd" );
-    papszFileList = CSLAddString( papszFileList, osHDRFilename );
+    CPLString osFilename = CPLFormCIFilename( osPath, osName, ".sgrd" );
+    papszFileList = CSLAddString( papszFileList, osFilename );
+
+    // projections file.
+    osFilename = CPLFormCIFilename( osPath, osName, "prj" );
+    if( VSIStatExL( osFilename, &sStatBuf, VSI_STAT_EXISTS_FLAG ) == 0 )
+        papszFileList = CSLAddString( papszFileList, osFilename );
     
     return papszFileList;
 }
 
+/************************************************************************/
+/*                          GetProjectionRef()                          */
+/************************************************************************/
+
+const char *SAGADataset::GetProjectionRef()
+
+{
+    if (pszProjection && strlen(pszProjection) > 0)
+        return pszProjection;
+
+    return GDALPamDataset::GetProjectionRef();
+}
+
+/************************************************************************/
+/*                           SetProjection()                            */
+/************************************************************************/
+
+CPLErr SAGADataset::SetProjection( const char *pszSRS )
+
+{
+/* -------------------------------------------------------------------- */
+/*      Reset coordinate system on the dataset.                         */
+/* -------------------------------------------------------------------- */
+    CPLFree( pszProjection );
+    pszProjection = CPLStrdup( pszSRS );
+
+    if( strlen(pszSRS) == 0 )
+        return CE_None;
+
+/* -------------------------------------------------------------------- */
+/*      Convert to ESRI WKT.                                            */
+/* -------------------------------------------------------------------- */
+    OGRSpatialReference oSRS( pszSRS );
+    char *pszESRI_SRS = NULL;
+
+    oSRS.morphToESRI();
+    oSRS.exportToWkt( &pszESRI_SRS );
+
+/* -------------------------------------------------------------------- */
+/*      Write to .prj file.                                             */
+/* -------------------------------------------------------------------- */
+    CPLString osPrjFilename = CPLResetExtension( GetDescription(), "prj" );
+    VSILFILE *fp;
+
+    fp = VSIFOpenL( osPrjFilename.c_str(), "wt" );
+    if( fp != NULL )
+    {
+        VSIFWriteL( pszESRI_SRS, 1, strlen(pszESRI_SRS), fp );
+        VSIFWriteL( (void *) "\n", 1, 1, fp );
+        VSIFCloseL( fp );
+    }
+
+    CPLFree( pszESRI_SRS );
+
+    return CE_None;
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -570,6 +645,31 @@ GDALDataset *SAGADataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->SetDescription( poOpenInfo->pszFilename );
     poDS->TryLoadXML();
 
+/* -------------------------------------------------------------------- */
+/*      Check for a .prj file.                                          */
+/* -------------------------------------------------------------------- */
+    const char  *pszPrjFilename = CPLFormCIFilename( osPath, osName, "prj" );
+
+    fp = VSIFOpenL( pszPrjFilename, "r" );
+
+    if( fp != NULL )
+    {
+        char  **papszLines;
+        OGRSpatialReference oSRS;
+
+        VSIFCloseL( fp );
+        
+        papszLines = CSLLoad( pszPrjFilename );
+
+        if( oSRS.importFromESRI( papszLines ) == OGRERR_NONE )
+        {
+            CPLFree( poDS->pszProjection );
+            oSRS.exportToWkt( &(poDS->pszProjection) );
+        }
+
+        CSLDestroy( papszLines );
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Check for external overviews.                                   */
 /* -------------------------------------------------------------------- */
@@ -964,6 +1064,8 @@ GDALDataset *SAGADataset::CreateCopy( const char *pszFilename,
 
     poSrcDS->GetGeoTransform( adfGeoTransform );
     poDstDS->SetGeoTransform( adfGeoTransform );
+    
+    poDstDS->SetProjection( poSrcDS->GetProjectionRef() );
 
     return poDstDS;
 }
