@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtdataset.cpp 22809 2011-07-24 21:51:09Z rouault $
+ * $Id: vrtdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTDataset
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2001, Frank Warmerdam <warmerdam@pobox.com>
+ * Copyright (c) 2007-2014, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +33,7 @@
 #include "cpl_minixml.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: vrtdataset.cpp 22809 2011-07-24 21:51:09Z rouault $");
+CPL_CVSID("$Id: vrtdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
 
 /************************************************************************/
 /*                            VRTDataset()                             */
@@ -58,7 +59,7 @@ VRTDataset::VRTDataset( int nXSize, int nYSize )
 
     nGCPCount = 0;
     pasGCPList = NULL;
-    pszGCPProjection = CPLStrdup("");
+    pszGCPProjection = NULL;
 
     pszVRTPath = NULL;
 
@@ -230,51 +231,10 @@ CPLXMLNode *VRTDataset::SerializeToXML( const char *pszVRTPath )
  /* -------------------------------------------------------------------- */
     if( nGCPCount > 0 )
     {
-        CPLXMLNode *psGCPList = CPLCreateXMLNode( psDSTree, CXT_Element, 
-                                                  "GCPList" );
-
-        CPLXMLNode* psLastChild = NULL;
-
-        if( pszGCPProjection != NULL && strlen(pszGCPProjection) > 0 )
-        {
-            CPLSetXMLValue( psGCPList, "#Projection", pszGCPProjection );
-            psLastChild = psGCPList->psChild;
-        }
-
-        for( int iGCP = 0; iGCP < nGCPCount; iGCP++ )
-        {
-            CPLXMLNode *psXMLGCP;
-            GDAL_GCP *psGCP = pasGCPList + iGCP;
-
-            psXMLGCP = CPLCreateXMLNode( NULL, CXT_Element, "GCP" );
-
-            if( psLastChild == NULL )
-                psGCPList->psChild = psXMLGCP;
-            else
-                psLastChild->psNext = psXMLGCP;
-            psLastChild = psXMLGCP;
-
-            CPLSetXMLValue( psXMLGCP, "#Id", psGCP->pszId );
-
-            if( psGCP->pszInfo != NULL && strlen(psGCP->pszInfo) > 0 )
-                CPLSetXMLValue( psXMLGCP, "Info", psGCP->pszInfo );
-
-            CPLSetXMLValue( psXMLGCP, "#Pixel", 
-                            CPLSPrintf( "%.4f", psGCP->dfGCPPixel ) );
-
-            CPLSetXMLValue( psXMLGCP, "#Line", 
-                            CPLSPrintf( "%.4f", psGCP->dfGCPLine ) );
-
-            CPLSetXMLValue( psXMLGCP, "#X", 
-                            CPLSPrintf( "%.12E", psGCP->dfGCPX ) );
-
-            CPLSetXMLValue( psXMLGCP, "#Y", 
-                            CPLSPrintf( "%.12E", psGCP->dfGCPY ) );
-
-            if( psGCP->dfGCPZ != 0.0 )
-                CPLSetXMLValue( psXMLGCP, "#GCPZ", 
-                                CPLSPrintf( "%.12E", psGCP->dfGCPZ ) );
-        }
+        GDALSerializeGCPListToXML( psDSTree,
+                                   pasGCPList,
+                                   nGCPCount,
+                                   pszGCPProjection );
     }
 
     /* -------------------------------------------------------------------- */
@@ -380,55 +340,12 @@ CPLErr VRTDataset::XMLInit( CPLXMLNode *psTree, const char *pszVRTPath )
 
     if( psGCPList != NULL )
     {
-        CPLXMLNode *psXMLGCP;
-        OGRSpatialReference oSRS;
-        const char *pszRawProj = CPLGetXMLValue(psGCPList, "Projection", "");
-
-        CPLFree( pszGCPProjection );
-
-        if( strlen(pszRawProj) > 0 
-            && oSRS.SetFromUserInput( pszRawProj ) == OGRERR_NONE )
-            oSRS.exportToWkt( &pszGCPProjection );
-        else
-            pszGCPProjection = CPLStrdup("");
-
-        // Count GCPs.
-        int  nGCPMax = 0;
-         
-        for( psXMLGCP = psGCPList->psChild; psXMLGCP != NULL; 
-             psXMLGCP = psXMLGCP->psNext )
-            nGCPMax++;
-         
-        pasGCPList = (GDAL_GCP *) CPLCalloc(sizeof(GDAL_GCP),nGCPMax);
-         
-        for( psXMLGCP = psGCPList->psChild; psXMLGCP != NULL; 
-             psXMLGCP = psXMLGCP->psNext )
-        {
-            GDAL_GCP *psGCP = pasGCPList + nGCPCount;
-
-            if( !EQUAL(psXMLGCP->pszValue,"GCP") || 
-                psXMLGCP->eType != CXT_Element )
-                continue;
-             
-            GDALInitGCPs( 1, psGCP );
-             
-            CPLFree( psGCP->pszId );
-            psGCP->pszId = CPLStrdup(CPLGetXMLValue(psXMLGCP,"Id",""));
-             
-            CPLFree( psGCP->pszInfo );
-            psGCP->pszInfo = CPLStrdup(CPLGetXMLValue(psXMLGCP,"Info",""));
-             
-            psGCP->dfGCPPixel = atof(CPLGetXMLValue(psXMLGCP,"Pixel","0.0"));
-            psGCP->dfGCPLine = atof(CPLGetXMLValue(psXMLGCP,"Line","0.0"));
-             
-            psGCP->dfGCPX = atof(CPLGetXMLValue(psXMLGCP,"X","0.0"));
-            psGCP->dfGCPY = atof(CPLGetXMLValue(psXMLGCP,"Y","0.0"));
-            psGCP->dfGCPZ = atof(CPLGetXMLValue(psXMLGCP,"Z","0.0"));
-
-            nGCPCount++;
-        }
+        GDALDeserializeGCPListFromXML( psGCPList,
+                                       &pasGCPList,
+                                       &nGCPCount,
+                                       &pszGCPProjection );
     }
-     
+
 /* -------------------------------------------------------------------- */
 /*      Apply any dataset level metadata.                               */
 /* -------------------------------------------------------------------- */
@@ -542,7 +459,10 @@ int VRTDataset::GetGCPCount()
 const char *VRTDataset::GetGCPProjection()
 
 {
-    return pszGCPProjection;
+    if( pszGCPProjection == NULL )
+        return "";
+    else
+        return pszGCPProjection;
 }
 
 /************************************************************************/
@@ -739,7 +659,53 @@ GDALDataset *VRTDataset::Open( GDALOpenInfo * poOpenInfo )
         }
         
         pszXML[nLength] = '\0';
-        pszVRTPath = CPLStrdup(CPLGetPath(poOpenInfo->pszFilename));
+
+        char* pszCurDir = CPLGetCurrentDir();
+        const char *currentVrtFilename = CPLProjectRelativeFilename(pszCurDir, poOpenInfo->pszFilename);
+        CPLFree(pszCurDir);
+#if defined(HAVE_READLINK) && defined(HAVE_LSTAT)
+        VSIStatBuf statBuffer;
+        char filenameBuffer[2048];
+
+        while( true ) {
+            int lstatCode = lstat( currentVrtFilename, &statBuffer );
+            if ( lstatCode == -1 ) {
+                if (errno == ENOENT) {
+                    // The file could be a virtual file, let later checks handle it.
+                    break;
+                } else {
+                    VSIFCloseL(fp);
+                    CPLFree( pszXML );
+                    CPLError( CE_Failure, CPLE_FileIO,
+                              "Failed to lstat %s: %s",
+                              currentVrtFilename,
+                              VSIStrerror(errno) );
+                    return NULL;
+                }
+            }
+
+            if ( !VSI_ISLNK(statBuffer.st_mode) ) {
+                break;
+            }
+
+            int bufferSize = readlink(currentVrtFilename, filenameBuffer, sizeof(filenameBuffer));
+            if (bufferSize != -1) {
+                filenameBuffer[MIN(bufferSize, (int) sizeof(filenameBuffer) - 1)] = 0;
+                // The filename in filenameBuffer might be a relative path from the linkfile resolve it before looping
+                currentVrtFilename = CPLProjectRelativeFilename(CPLGetDirname(currentVrtFilename), filenameBuffer);
+            } else {
+                VSIFCloseL(fp);
+                CPLFree( pszXML );
+                CPLError( CE_Failure, CPLE_FileIO,
+                          "Failed to read filename from symlink %s: %s",
+                          currentVrtFilename,
+                          VSIStrerror(errno) );
+                return NULL;
+            }
+        }
+#endif
+
+        pszVRTPath = CPLStrdup(CPLGetPath(currentVrtFilename));
 
         VSIFCloseL(fp);
     }
@@ -877,7 +843,8 @@ CPLErr VRTDataset::AddBand( GDALDataType eType, char **papszOptions )
 /*      Collect required information.                                   */
 /* -------------------------------------------------------------------- */
         if( CSLFetchNameValue(papszOptions, "ImageOffset") != NULL )
-            nImageOffset = atoi(CSLFetchNameValue(papszOptions, "ImageOffset"));
+            nImageOffset = CPLScanUIntBig(
+                CSLFetchNameValue(papszOptions, "ImageOffset"), 20);
 
         if( CSLFetchNameValue(papszOptions, "PixelOffset") != NULL )
             nPixelOffset = atoi(CSLFetchNameValue(papszOptions,"PixelOffset"));
@@ -898,7 +865,7 @@ CPLErr VRTDataset::AddBand( GDALDataType eType, char **papszOptions )
         }
         
         bRelativeToVRT = 
-            CSLFetchBoolean( papszOptions, "RelativeToVRT", FALSE );
+            CSLFetchBoolean( papszOptions, "relativeToVRT", FALSE );
 
 /* -------------------------------------------------------------------- */
 /*      Create and initialize the band.                                 */
@@ -909,7 +876,7 @@ CPLErr VRTDataset::AddBand( GDALDataType eType, char **papszOptions )
             new VRTRawRasterBand( this, GetRasterCount() + 1, eType );
 
         eErr = 
-            poBand->SetRawLink( pszFilename, NULL, FALSE, 
+            poBand->SetRawLink( pszFilename, NULL, bRelativeToVRT,
                                 nImageOffset, nPixelOffset, nLineOffset, 
                                 pszByteOrder );
         if( eErr != CE_None )
@@ -1139,7 +1106,7 @@ CPLErr VRTDataset::Delete( const char * pszFilename )
 /*                          CreateMaskBand()                            */
 /************************************************************************/
 
-CPLErr VRTDataset::CreateMaskBand( int nFlags )
+CPLErr VRTDataset::CreateMaskBand( CPL_UNUSED int nFlags )
 {
     if (poMaskBand != NULL)
     {
@@ -1252,6 +1219,53 @@ int VRTDataset::CheckCompatibleForDatasetIO()
     return nSources != 0;
 }
 
+/************************************************************************/
+/*                         GetSingleSimpleSource()                      */
+/*                                                                      */
+/* Returns a non-NULL dataset if the VRT is made of a single source     */
+/* that is a simple source, in its full extent, and with all of its     */
+/* bands. Basically something produced by :                             */
+/*   gdal_translate src dst.vrt -of VRT (-a_srs / -a_ullr)              */
+/************************************************************************/
+
+GDALDataset* VRTDataset::GetSingleSimpleSource()
+{
+    if (!CheckCompatibleForDatasetIO())
+        return NULL;
+
+    VRTSourcedRasterBand* poVRTBand = (VRTSourcedRasterBand* )papoBands[0];
+    VRTSimpleSource* poSource = (VRTSimpleSource* )poVRTBand->papoSources[0];
+    GDALRasterBand* poBand = poSource->GetBand();
+    if (poBand == NULL)
+        return NULL;
+    GDALDataset* poSrcDS = poBand->GetDataset();
+    if (poSrcDS == NULL)
+        return NULL;
+
+    /* Check that it uses the full source dataset */
+    int nReqXOff, nReqYOff, nReqXSize, nReqYSize;
+    int nOutXOff, nOutYOff, nOutXSize, nOutYSize;
+    poSource->GetSrcDstWindow( 0, 0,
+                               poSrcDS->GetRasterXSize(), poSrcDS->GetRasterYSize(),
+                               poSrcDS->GetRasterXSize(), poSrcDS->GetRasterYSize(),
+                               &nReqXOff, &nReqYOff,
+                               &nReqXSize, &nReqYSize,
+                               &nOutXOff, &nOutYOff,
+                               &nOutXSize, &nOutYSize );
+
+    if (nReqXOff != 0 || nReqYOff != 0 ||
+        nReqXSize != poSrcDS->GetRasterXSize() ||
+        nReqYSize != poSrcDS->GetRasterYSize())
+        return NULL;
+
+    if (nOutXOff != 0 || nOutYOff != 0 ||
+        nOutXSize != poSrcDS->GetRasterXSize() ||
+        nOutYSize != poSrcDS->GetRasterYSize())
+        return NULL;
+
+    return poSrcDS;
+}
+
 /************************************************************************/
 /*                              IRasterIO()                             */
 /************************************************************************/
@@ -1267,7 +1281,7 @@ CPLErr VRTDataset::IRasterIO( GDALRWFlag eRWFlag,
     {
         bCompatibleForDatasetIO = CheckCompatibleForDatasetIO();
     }
-    if (bCompatibleForDatasetIO && eRWFlag == GF_Read && nBandCount > 1)
+    if (bCompatibleForDatasetIO && eRWFlag == GF_Read)
     {
         for(int iBandIndex=0; iBandIndex<nBandCount; iBandIndex++)
         {
