@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_xml.cpp 25019 2012-09-30 13:25:39Z rouault $
+ * $Id: ogr_srs_xml.cpp 27729 2014-09-24 00:40:16Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference interface to OGC XML (014r4).
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2001, Frank Warmerdam (warmerdam@pobox.com)
+ * Copyright (c) 2008-2012, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -713,8 +714,7 @@ static CPLXMLNode *exportProjCSToXML( const OGRSpatialReference *poSRS )
  */
 
 OGRErr OGRSpatialReference::exportToXML( char **ppszRawXML, 
-                                         const char * pszDialect ) const
-
+                                         CPL_UNUSED const char * pszDialect ) const
 {
     CPLXMLNode *psXMLTree = NULL;
 
