@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: ogrgftlayer.cpp 22268 2011-04-30 13:10:40Z rouault $
+ * $Id: ogrgftlayer.cpp 27044 2014-03-16 23:41:27Z rouault $
  *
  * Project:  GFT Translator
  * Purpose:  Implements OGRGFTLayer class.
  * Author:   Even Rouault, <even dot rouault at mines dash paris dot org>
  *
  ******************************************************************************
- * Copyright (c) 2011, Even Rouault <even dot rouault at mines dash paris dot org>
+ * Copyright (c) 2011-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,7 @@
 #include "ogr_gft.h"
 #include "cpl_minixml.h"
 
-CPL_CVSID("$Id: ogrgftlayer.cpp 22268 2011-04-30 13:10:40Z rouault $");
+CPL_CVSID("$Id: ogrgftlayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
 
 /************************************************************************/
 /*                            OGRGFTLayer()                             */
@@ -524,25 +524,6 @@ int OGRGFTLayer::TestCapability( const char * pszCap )
     return FALSE;
 }
 
-/************************************************************************/
-/*                         GetGeometryColumn()                          */
-/************************************************************************/
-
-const char *OGRGFTLayer::GetGeometryColumn()
-{
-    GetLayerDefn();
-    if (iGeometryField < 0)
-        return "";
-
-    if (iGeometryField == poFeatureDefn->GetFieldCount())
-    {
-        CPLAssert(bHiddenGeometryField);
-        return GetDefaultGeometryColumnName();
-    }
-
-    return poFeatureDefn->GetFieldDefn(iGeometryField)->GetNameRef();
-}
-
 /************************************************************************/
 /*                         ParseCSVResponse()                           */
 /************************************************************************/
@@ -647,20 +628,6 @@ CPLString OGRGFTLayer::PatchSQL(const char* pszSQL)
     return osSQL;
 }
 
-/************************************************************************/
-/*                          GetSpatialRef()                             */
-/************************************************************************/
-
-OGRSpatialReference* OGRGFTLayer::GetSpatialRef()
-{
-    GetLayerDefn();
-
-    if (iGeometryField < 0)
-        return NULL;
-
-    return poSRS;
-}
-
 /************************************************************************/
 /*                         LaunderColName()                             */
 /************************************************************************/
@@ -678,3 +645,23 @@ CPLString OGRGFTLayer::LaunderColName(const char* pszColName)
     }
     return osLaunderedColName;
 }
+
+/************************************************************************/
+/*                         SetGeomFieldName()                           */
+/************************************************************************/
+
+void OGRGFTLayer::SetGeomFieldName()
+{
+    if (iGeometryField >= 0 && poFeatureDefn->GetGeomFieldCount() > 0)
+    {
+        const char* pszGeomColName;
+        if (iGeometryField == poFeatureDefn->GetFieldCount())
+        {
+            CPLAssert(bHiddenGeometryField);
+            pszGeomColName = GetDefaultGeometryColumnName();
+        }
+        else
+            pszGeomColName = poFeatureDefn->GetFieldDefn(iGeometryField)->GetNameRef();
+        poFeatureDefn->GetGeomFieldDefn(0)->SetName(pszGeomColName);
+    }
+}
