@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrili1datasource.cpp 23890 2012-02-03 16:30:23Z pka $
+ * $Id: ogrili1datasource.cpp 27044 2014-03-16 23:41:27Z rouault $
  *
  * Project:  Interlis 1 Translator
  * Purpose:  Implements OGRILI1DataSource class.
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Pirmin Kalberer, Sourcepole AG
+ * Copyright (c) 2007-2008, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -33,12 +34,9 @@
 
 #include "ili1reader.h"
 
-#include "iomhelper.h"
-#include "iom/iom.h"
-
 #include <string>
 
-CPL_CVSID("$Id: ogrili1datasource.cpp 23890 2012-02-03 16:30:23Z pka $");
+CPL_CVSID("$Id: ogrili1datasource.cpp 27044 2014-03-16 23:41:27Z rouault $");
 
 /************************************************************************/
 /*                         OGRILI1DataSource()                         */
@@ -48,6 +46,7 @@ OGRILI1DataSource::OGRILI1DataSource()
 
 {
     pszName = NULL;
+    poImdReader = new ImdReader(1);
     poReader = NULL;
     fpTransfer = NULL;
     pszTopic = NULL;
@@ -73,6 +72,7 @@ OGRILI1DataSource::~OGRILI1DataSource()
     CPLFree( pszName );
     CPLFree( pszTopic );
     DestroyILI1Reader( poReader );
+    delete poImdReader;
     if( fpTransfer )
     {
         VSIFPrintf( fpTransfer, "ETAB\n" );
@@ -163,7 +163,7 @@ int OGRILI1DataSource::Open( const char * pszNewName, int bTestOpen )
     pszName = CPLStrdup( osBasename.c_str() );
 
     if (osModelFilename.length() > 0 )
-        poReader->ReadModel( osModelFilename.c_str() );
+        poReader->ReadModel( poImdReader, osModelFilename.c_str(), this );
 
     if( getenv( "ARC_DEGREES" ) != NULL ) {
       //No better way to pass arguments to the reader (it could even be an -lco arg)
@@ -194,11 +194,6 @@ int OGRILI1DataSource::Create( const char *pszFilename,
 
     CSLDestroy( filenames );
 
-    if( osModelFilename.length() == 0 )
-    {
-      //TODO: create automatic model
-    }
-
 /* -------------------------------------------------------------------- */
 /*      Create the empty file.                                          */
 /* -------------------------------------------------------------------- */
@@ -217,29 +212,15 @@ int OGRILI1DataSource::Create( const char *pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Parse model                                                     */
 /* -------------------------------------------------------------------- */
-    iom_init();
-
-    // set error listener to a iom provided one, that just
-    // dumps all errors to stderr
-    iom_seterrlistener(iom_stderrlistener);
-
-    IOM_BASKET model = 0;
-    if( osModelFilename.length() != 0 ) {
-      // compile ili model
-      char *iliFiles[1] = {(char *)osModelFilename.c_str()};
-      model=iom_compileIli(1,iliFiles);
-      if(!model){
-        CPLError( CE_Warning, CPLE_OpenFailed,
-                  "iom_compileIli %s, %s.",
-                  pszName, VSIStrerror( errno ) );
-        iom_end();
-        return FALSE;
-      }
+    if( osModelFilename.length() == 0 )
+    {
+        CPLError(CE_Warning, CPLE_AppDefined,
+            "Creating Interlis transfer file without model definition." );
+    } else {
+        poImdReader->ReadModel(osModelFilename.c_str());
     }
 
-    pszTopic = CPLStrdup(model ?
-                         GetAttrObjName(model, "iom04.metamodel.Topic") :
-                         CPLGetBasename(osBasename.c_str()));
+    pszTopic = CPLStrdup(poImdReader->mainTopicName.c_str());
 
 /* -------------------------------------------------------------------- */
 /*      Write headers                                                   */
@@ -248,9 +229,7 @@ int OGRILI1DataSource::Create( const char *pszFilename,
     VSIFPrintf( fpTransfer, "OGR/GDAL %s, INTERLIS Driver\n", GDAL_RELEASE_NAME );
     VSIFPrintf( fpTransfer, "////\n" );
     VSIFPrintf( fpTransfer, "MTID INTERLIS1\n" );
-    const char* modelname = model ?
-                            GetAttrObjName(model, "iom04.metamodel.DataModel") :
-                            CPLGetBasename(osBasename.c_str());
+    const char* modelname = poImdReader->mainModelName.c_str();
     VSIFPrintf( fpTransfer, "MODL %s\n", modelname );
 
     return TRUE;
@@ -274,6 +253,7 @@ OGRILI1DataSource::CreateLayer( const char * pszLayerName,
                                char ** papszOptions )
 
 {
+    FeatureDefnInfo featureDefnInfo = poImdReader->GetFeatureDefnInfo(pszLayerName);
     const char *table = pszLayerName;
     char * topic = ExtractTopic(pszLayerName);
     if (nLayers) VSIFPrintf( fpTransfer, "ETAB\n" );
@@ -302,7 +282,9 @@ OGRILI1DataSource::CreateLayer( const char * pszLayerName,
     }
     VSIFPrintf( fpTransfer, "TABL %s\n", table );
 
-    OGRILI1Layer *poLayer = new OGRILI1Layer(table, poSRS, TRUE, eType, this);
+    OGRFeatureDefn* poFeatureDefn = new OGRFeatureDefn(table);
+    poFeatureDefn->SetGeomType( eType );
+    OGRILI1Layer *poLayer = new OGRILI1Layer(poFeatureDefn, featureDefnInfo.poGeomFieldInfos, this);
 
     nLayers ++;
     papoLayers = (OGRILI1Layer**)CPLRealloc(papoLayers, sizeof(OGRILI1Layer*) * nLayers);
@@ -332,3 +314,12 @@ OGRLayer *OGRILI1DataSource::GetLayer( int iLayer )
 {
   return poReader->GetLayer( iLayer );
 }
+
+/************************************************************************/
+/*                              GetLayerByName()                              */
+/************************************************************************/
+
+OGRILI1Layer *OGRILI1DataSource::GetLayerByName( const char* pszLayerName )
+{
+  return (OGRILI1Layer*)poReader->GetLayerByName( pszLayerName );
+}
