@@ -11,6 +11,7 @@
  *
  **********************************************************************
  * Copyright (c) 1999-2003, Stephane Villeneuve
+ * Copyright (c) 2011-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -270,7 +271,7 @@ int MIFFile::Open(const char *pszFname, const char *pszAccess,
     
     CPLErrorReset();
 
-    if (m_poMIDFile)
+    if (m_poMIFFile)
     {
         CPLError(CE_Failure, CPLE_FileIO,
                      "Open() failed: object already contains an open file");
@@ -360,38 +361,6 @@ int MIFFile::Open(const char *pszFname, const char *pszAccess,
         return -1;
     }
 
-    /*-----------------------------------------------------------------
-     * Open .MID file
-     *----------------------------------------------------------------*/
-    if (nFnameLen > 4 && strcmp(pszTmpFname+nFnameLen-4, ".MIF")==0)
-        strcpy(pszTmpFname+nFnameLen-4, ".MID");
-    else 
-        strcpy(pszTmpFname+nFnameLen-4, ".mid");
-
-#ifndef _WIN32
-    TABAdjustFilenameExtension(pszTmpFname);
-#endif
-
-    m_poMIDFile = new MIDDATAFile;
-
-    if (m_poMIDFile->Open(pszTmpFname, pszAccess) !=0)
-    {
-        if (!bTestOpenNoError)
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "Unable to open %s.", pszTmpFname);
-        else
-            CPLErrorReset();
-
-        CPLFree(pszTmpFname);
-        Close();
-
-        return -1;
-    }
-
-
-    CPLFree(pszTmpFname);
-    pszTmpFname = NULL;
-
     /*-----------------------------------------------------------------
      * Read MIF File Header
      *----------------------------------------------------------------*/
@@ -405,9 +374,45 @@ int MIFFile::Open(const char *pszFname, const char *pszAccess,
         else
             CPLErrorReset();
 
+        CPLFree(pszTmpFname);
+
         return -1;
     }
 
+    if ( m_nAttribut > 0 || m_eAccessMode == TABWrite )
+    {
+        /*-----------------------------------------------------------------
+        * Open .MID file
+        *----------------------------------------------------------------*/
+        if (nFnameLen > 4 && strcmp(pszTmpFname+nFnameLen-4, ".MIF")==0)
+            strcpy(pszTmpFname+nFnameLen-4, ".MID");
+        else 
+            strcpy(pszTmpFname+nFnameLen-4, ".mid");
+
+#ifndef _WIN32
+        TABAdjustFilenameExtension(pszTmpFname);
+#endif
+
+        m_poMIDFile = new MIDDATAFile;
+
+        if (m_poMIDFile->Open(pszTmpFname, pszAccess) !=0)
+        {
+            if (!bTestOpenNoError)
+                CPLError(CE_Failure, CPLE_NotSupported,
+                        "Unable to open %s.", pszTmpFname);
+            else
+                CPLErrorReset();
+
+            CPLFree(pszTmpFname);
+            Close();
+
+            return -1;
+        }
+    }
+
+    CPLFree(pszTmpFname);
+    pszTmpFname = NULL;
+
     /*-----------------------------------------------------------------
      * In write access, set some defaults
      *----------------------------------------------------------------*/
@@ -418,7 +423,7 @@ int MIFFile::Open(const char *pszFname, const char *pszAccess,
     }
 
     /* Put the MID file at the correct location, on the first feature */
-    if (m_eAccessMode == TABRead && (m_poMIDFile->GetLine() == NULL))
+    if (m_eAccessMode == TABRead && (m_poMIDFile != NULL && m_poMIDFile->GetLine() == NULL))
     {
         Close();
 
@@ -430,10 +435,12 @@ int MIFFile::Open(const char *pszFname, const char *pszAccess,
 
     m_poMIFFile->SetTranslation(m_dfXMultiplier,m_dfYMultiplier,
                                 m_dfXDisplacement, m_dfYDisplacement);
-    m_poMIDFile->SetTranslation(m_dfXMultiplier,m_dfYMultiplier,
-                                m_dfXDisplacement, m_dfYDisplacement);
+    if( m_poMIDFile != NULL )
+        m_poMIDFile->SetTranslation(m_dfXMultiplier,m_dfYMultiplier,
+                                    m_dfXDisplacement, m_dfYDisplacement);
     m_poMIFFile->SetDelimiter(m_pszDelimiter);
-    m_poMIDFile->SetDelimiter(m_pszDelimiter);
+    if( m_poMIDFile != NULL )
+        m_poMIDFile->SetDelimiter(m_pszDelimiter);
 
     /*-------------------------------------------------------------
      * Set geometry type if the geometry objects are uniform.
@@ -858,8 +865,11 @@ void MIFFile::ResetReading()
           break;
     }
 
-    m_poMIDFile->Rewind();
-    m_poMIDFile->GetLine();
+    if( m_poMIDFile != NULL )
+    {
+        m_poMIDFile->Rewind();
+        m_poMIDFile->GetLine();
+    }
     
     // We're positioned on first feature.  Feature Ids start at 1.
     if (m_poCurFeature)
@@ -984,8 +994,11 @@ void MIFFile::PreParseFile()
           break;
     }
 
-    m_poMIDFile->Rewind();
-    m_poMIDFile->GetLine();
+    if( m_poMIDFile != NULL )
+    {
+        m_poMIDFile->Rewind();
+        m_poMIDFile->GetLine();
+    }
  
     m_bPreParsed = TRUE;
 
@@ -1086,13 +1099,8 @@ int MIFFile::WriteMIFHeader()
         switch(m_paeFieldType[iField])
         {
           case TABFInteger:
-            if (poFieldDefn->GetWidth() == 0)
-                m_poMIFFile->WriteLine("  %s Integer\n",
-                                   poFieldDefn->GetNameRef());
-            else
-                m_poMIFFile->WriteLine("  %s Integer(%d)\n",
-                                   poFieldDefn->GetNameRef(),
-                                   poFieldDefn->GetWidth());
+            m_poMIFFile->WriteLine("  %s Integer\n",
+                                poFieldDefn->GetNameRef());
             break;
           case TABFSmallInt:
             m_poMIFFile->WriteLine("  %s SmallInt\n",
@@ -1291,7 +1299,8 @@ GBool MIFFile::NextFeature()
     {
         if (m_poMIFFile->IsValidFeature(pszLine))
         {
-            m_poMIDFile->GetLine();
+            if( m_poMIDFile != NULL )
+                m_poMIDFile->GetLine();
             m_nPreloadedId++;
             return TRUE;
         }
@@ -1328,7 +1337,7 @@ TABFeature *MIFFile::GetFeatureRef(int nFeatureId)
      * Make sure file is opened and Validate feature id by positioning
      * the read pointers for the .MAP and .DAT files to this feature id.
      *----------------------------------------------------------------*/
-    if (m_poMIDFile == NULL)
+    if (m_poMIFFile == NULL)
     {
         CPLError(CE_Failure, CPLE_IllegalArg,
                  "GetFeatureRef() failed: file is not opened!");
@@ -1468,7 +1477,7 @@ TABFeature *MIFFile::GetFeatureRef(int nFeatureId)
      * Read fields from the .DAT file
      * GetRecordBlock() has already been called above...
      *----------------------------------------------------------------*/
-    if (m_poCurFeature->ReadRecordFromMIDFile(m_poMIDFile) != 0)
+    if (m_poMIDFile != NULL && m_poCurFeature->ReadRecordFromMIDFile(m_poMIDFile) != 0)
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "Error during reading Record.");
@@ -1759,7 +1768,7 @@ int MIFFile::AddFieldNative(const char *pszName, TABFieldType eMapInfoType,
      *----------------------------------------------------------------*/
     if (eMapInfoType == TABFDecimal && nWidth == 0)
         nWidth=20;
-    else if (nWidth == 0)
+    else if (eMapInfoType == TABFChar && nWidth == 0)
         nWidth=254; /* char fields */
 
     /*-----------------------------------------------------------------
@@ -2241,9 +2250,6 @@ int MIFFile::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCSequentialWrite) )
         return TRUE;
 
-    else if( EQUAL(pszCap,OLCSequentialWrite) )
-        return FALSE;
-
     else if( EQUAL(pszCap,OLCFastFeatureCount) )
         return m_bPreParsed;
 
@@ -2253,6 +2259,9 @@ int MIFFile::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCFastGetExtent) )
         return m_bPreParsed;
 
+    else if( EQUAL(pszCap,OLCCreateField) ) 
+        return TRUE; 
+
     else 
         return FALSE;
 }
