@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqltablelayer.cpp 25513 2013-01-16 19:04:39Z rouault $
+ * $Id: ogrmysqltablelayer.cpp 27044 2014-03-16 23:41:27Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLTableLayer class.
@@ -8,6 +8,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam <warmerdam@pobox.com>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +33,7 @@
 #include "cpl_string.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqltablelayer.cpp 25513 2013-01-16 19:04:39Z rouault $");
+CPL_CVSID("$Id: ogrmysqltablelayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
 
 /************************************************************************/
 /*                         OGRMySQLTableLayer()                         */
@@ -411,15 +412,15 @@ void OGRMySQLTableLayer::BuildWhere()
 
     if( m_poFilterGeom != NULL && pszGeomColumn )
     {
-        char szEnvelope[4096];
+        char szEnvelope[400];
         OGREnvelope  sEnvelope;
         szEnvelope[0] = '\0';
         
         //POLYGON((MINX MINY, MAXX MINY, MAXX MAXY, MINX MAXY, MINX MINY))
         m_poFilterGeom->getEnvelope( &sEnvelope );
         
-        sprintf(szEnvelope,
-                "POLYGON((%.12f %.12f, %.12f %.12f, %.12f %.12f, %.12f %.12f, %.12f %.12f))",
+        snprintf(szEnvelope, sizeof(szEnvelope),
+                "POLYGON((%.18g %.18g, %.18g %.18g, %.18g %.18g, %.18g %.18g, %.18g %.18g))",
                 sEnvelope.MinX, sEnvelope.MinY,
                 sEnvelope.MaxX, sEnvelope.MinY,
                 sEnvelope.MaxX, sEnvelope.MaxY,
@@ -547,6 +548,9 @@ char *OGRMySQLTableLayer::BuildFields()
 OGRErr OGRMySQLTableLayer::SetAttributeFilter( const char *pszQuery )
 
 {
+    CPLFree(m_pszAttrQueryString);
+    m_pszAttrQueryString = (pszQuery) ? CPLStrdup(pszQuery) : NULL;
+
     CPLFree( this->pszQuery );
 
     if( pszQuery == NULL || strlen(pszQuery) == 0 )
