@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrnasdatasource.cpp 25553 2013-01-26 11:45:56Z rouault $
+ * $Id: ogrnasdatasource.cpp 27713 2014-09-21 15:51:47Z jef $
  *
  * Project:  OGR
  * Purpose:  Implements OGRNASDataSource class.
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2002, Frank Warmerdam <warmerdam@pobox.com>
+ * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,7 +32,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrnasdatasource.cpp 25553 2013-01-26 11:45:56Z rouault $");
+CPL_CVSID("$Id: ogrnasdatasource.cpp 27713 2014-09-21 15:51:47Z jef $");
 
 static const char *apszURNNames[] =
 {
@@ -135,8 +136,11 @@ int OGRNASDataSource::Open( const char * pszNewName, int bTestOpen )
         if( szPtr[0] != '<'
             || strstr(szPtr,"opengis.net/gml") == NULL
             || (strstr(szPtr,"NAS-Operationen.xsd") == NULL &&
+                strstr(szPtr,"NAS-Operationen_optional.xsd") == NULL &&
                 strstr(szPtr,"AAA-Fachschema.xsd") == NULL ) )
         {
+            /*CPLDebug( "NAS",
+                      "Skipping. No chevrons of NAS found [%s]\n", szPtr );*/
             VSIFClose( fp );
             return FALSE;
         }
@@ -264,11 +268,15 @@ OGRNASLayer *OGRNASDataSource::TranslateNASSchema( GMLFeatureClass *poClass )
 
 {
     OGRNASLayer *poLayer;
-    OGRwkbGeometryType eGType
-        = (OGRwkbGeometryType) poClass->GetGeometryType();
+    OGRwkbGeometryType eGType = wkbNone;
 
-    if( poClass->GetFeatureCount() == 0 )
-        eGType = wkbUnknown;
+    if( poClass->GetGeometryPropertyCount() != 0 )
+    {
+        eGType = (OGRwkbGeometryType) poClass->GetGeometryProperty(0)->GetType();
+
+        if( poClass->GetFeatureCount() == 0 )
+            eGType = wkbUnknown;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Translate SRS.                                                  */
