@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpglayer.cpp 24545 2012-06-07 21:28:38Z rouault $
+ * $Id: ogrpglayer.cpp 27106 2014-03-28 11:59:57Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGLayer class  which implements shared handling
@@ -9,6 +9,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2000, Frank Warmerdam
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -64,7 +65,7 @@ PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 
 #define PQexec this_is_an_error
 
-CPL_CVSID("$Id: ogrpglayer.cpp 24545 2012-06-07 21:28:38Z rouault $");
+CPL_CVSID("$Id: ogrpglayer.cpp 27106 2014-03-28 11:59:57Z rouault $");
 
 #define CURSOR_PAGE     500
 
@@ -75,12 +76,6 @@ CPL_CVSID("$Id: ogrpglayer.cpp 24545 2012-06-07 21:28:38Z rouault $");
 #define INV_READ                0x00040000
 #endif
 
-/* Flags for creating WKB format for PostGIS */
-#define WKBZOFFSET 0x80000000
-#define WKBMOFFSET 0x40000000
-#define WKBSRIDFLAG 0x20000000
-#define WKBBBOXFLAG 0x10000000
-
 /************************************************************************/
 /*                           OGRPGLayer()                               */
 /************************************************************************/
@@ -90,11 +85,7 @@ OGRPGLayer::OGRPGLayer()
 {
     poDS = NULL;
 
-    bHasWkb = FALSE;
     bWkbAsOid = FALSE;
-    bHasPostGISGeometry = FALSE;
-    bHasPostGISGeography = FALSE;
-    pszGeomColumn = NULL;
     pszQueryStatement = NULL;
 
     bHasFid = FALSE;
@@ -103,19 +94,15 @@ OGRPGLayer::OGRPGLayer()
     iNextShapeId = 0;
     nResultOffset = 0;
 
-    nCoordDimension = 2; // initialize in case PostGIS is not available
-
-    poSRS = NULL;
-    nSRSId = UNDETERMINED_SRID; // we haven't even queried the database for it yet.
-
     pszCursorName = CPLStrdup(CPLSPrintf("OGRPGLayerReader%p", this));
-    
+
     hCursorResult = NULL;
 
     bCanUseBinaryCursor = TRUE;
 
     poFeatureDefn = NULL;
     panMapFieldNameToIndex = NULL;
+    panMapFieldNameToGeomIndex = NULL;
 }
 
 /************************************************************************/
@@ -134,17 +121,17 @@ OGRPGLayer::~OGRPGLayer()
 
     ResetReading();
 
-    CPLFree( pszGeomColumn );
     CPLFree( pszFIDColumn );
     CPLFree( pszQueryStatement );
     CPLFree( panMapFieldNameToIndex );
+    CPLFree( panMapFieldNameToGeomIndex );
     CPLFree( pszCursorName );
 
-    if( poSRS != NULL )
-        poSRS->Release();
-
     if( poFeatureDefn )
+    {
+        poFeatureDefn->UnsetLayer();
         poFeatureDefn->Release();
+    }
 }
 
 /************************************************************************/
@@ -635,11 +622,16 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
 /* -------------------------------------------------------------------- */
 /*      Handle PostGIS geometry                                         */
 /* -------------------------------------------------------------------- */
-
-        if( bHasPostGISGeometry || bHasPostGISGeography )
+        int iOGRGeomField = panMapFieldNameToGeomIndex[iField];
+        OGRPGGeomFieldDefn* poGeomFieldDefn = NULL;
+        if( iOGRGeomField >= 0 )
+            poGeomFieldDefn = poFeatureDefn->myGetGeomFieldDefn(iOGRGeomField);
+        if( iOGRGeomField >= 0 && (
+                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOMETRY ||
+                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOGRAPHY) )
         {
             if ( !poDS->bUseBinaryCursor &&
-                 EQUAL(pszFieldName,"BinaryBase64") )
+                 EQUALN(pszFieldName,"BinaryBase64", strlen("BinaryBase64")) )
             {
                 GByte* pabyData = (GByte*)PQgetvalue( hCursorResult,
                                                         iRecord, iField);
@@ -656,14 +648,14 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
 
                 if( poGeom != NULL )
                 {
-                    poGeom->assignSpatialReference( poSRS );
-                    poFeature->SetGeometryDirectly( poGeom );
+                    poGeom->assignSpatialReference( poGeomFieldDefn->GetSpatialRef() );
+                    poFeature->SetGeomFieldDirectly(iOGRGeomField,  poGeom );
                 }
 
                 continue;
             }
-            else if ( EQUAL(pszFieldName,"ST_AsBinary") ||
-                 EQUAL(pszFieldName,"AsBinary") )
+            else if ( EQUALN(pszFieldName,"ST_AsBinary", strlen("ST_AsBinary")) ||
+                      EQUALN(pszFieldName,"AsBinary", strlen("AsBinary")) )
             {
                 GByte* pabyVal = (GByte*) PQgetvalue( hCursorResult,
                                              iRecord, iField);
@@ -689,14 +681,14 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
                 
                 if( poGeom != NULL )
                 {
-                    poGeom->assignSpatialReference( poSRS );
-                    poFeature->SetGeometryDirectly( poGeom );
+                    poGeom->assignSpatialReference( poGeomFieldDefn->GetSpatialRef() );
+                    poFeature->SetGeomFieldDirectly(iOGRGeomField,  poGeom );
                 }
 
                 continue;
             }
             else if ( !poDS->bUseBinaryCursor &&
-                      EQUAL(pszFieldName,"EWKBBase64") )
+                      EQUALN(pszFieldName,"EWKBBase64",strlen("EWKBBase64")) )
             {
                 GByte* pabyData = (GByte*)PQgetvalue( hCursorResult,
                                                         iRecord, iField);
@@ -708,18 +700,17 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
                     continue;
 
                 nLength = CPLBase64DecodeInPlace(pabyData);
-                OGRGeometry * poGeom = EWKBToGeometry(pabyData, nLength);
+                OGRGeometry * poGeom = OGRGeometryFromEWKB(pabyData, nLength, NULL);
 
                 if( poGeom != NULL )
                 {
-                    poGeom->assignSpatialReference( poSRS );
-                    poFeature->SetGeometryDirectly( poGeom );
+                    poGeom->assignSpatialReference( poGeomFieldDefn->GetSpatialRef() );
+                    poFeature->SetGeomFieldDirectly(iOGRGeomField,  poGeom );
                 }
 
                 continue;
             }
-            else if ( (poDS->bUseBinaryCursor &&
-                       (EQUAL(pszFieldName,pszGeomColumn))) ||
+            else if ( poDS->bUseBinaryCursor ||
                       EQUAL(pszFieldName,"ST_AsEWKB") ||
                       EQUAL(pszFieldName,"AsEWKB") )
             {
@@ -740,31 +731,30 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
                      strncmp(pabyData, "\\000",4) == 0 || strncmp(pabyData, "\\001",4) == 0) )
                 {
                     GByte* pabyEWKB = BYTEAToGByteArray(pabyData, &nLength);
-                    poGeom = EWKBToGeometry(pabyEWKB, nLength);
+                    poGeom = OGRGeometryFromEWKB(pabyEWKB, nLength, NULL);
                     CPLFree(pabyEWKB);
                 }
                 else if( nLength >= 2 && (EQUALN(pabyData,"00",2) || EQUALN(pabyData,"01",2)) )
                 {
-                    poGeom = HEXToGeometry(pabyData);
+                    poGeom = OGRGeometryFromHexEWKB(pabyData, NULL);
                 }
                 else
                 {
-                    poGeom = EWKBToGeometry((GByte*)pabyData, nLength);
+                    poGeom = OGRGeometryFromEWKB((GByte*)pabyData, nLength, NULL);
                 }
 
                 if( poGeom != NULL )
                 {
-                    poGeom->assignSpatialReference( poSRS );
-                    poFeature->SetGeometryDirectly( poGeom );
+                    poGeom->assignSpatialReference( poGeomFieldDefn->GetSpatialRef() );
+                    poFeature->SetGeomFieldDirectly(iOGRGeomField,  poGeom );
                 }
 
                 continue;
             }
-            else if (EQUAL(pszFieldName,pszGeomColumn) ||
-                     EQUAL(pszFieldName,"asEWKT") ||
+            else /*if (EQUAL(pszFieldName,"asEWKT") ||
                      EQUAL(pszFieldName,"asText") ||
                      EQUAL(pszFieldName,"ST_AsEWKT") ||
-                     EQUAL(pszFieldName,"ST_AsText") )
+                     EQUAL(pszFieldName,"ST_AsText") )*/
             {
                 /* Handle WKT */
                 char        *pszWKT;
@@ -786,15 +776,15 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
 
                 if( EQUALN(pszPostSRID,"00",2) || EQUALN(pszPostSRID,"01",2) )
                 {
-                    poGeometry = HEXToGeometry( pszWKT );
+                    poGeometry = OGRGeometryFromHexEWKB( pszWKT, NULL );
                 }
                 else
                     OGRGeometryFactory::createFromWkt( &pszPostSRID, NULL,
                                                     &poGeometry );
                 if( poGeometry != NULL )
                 {
-                    poGeometry->assignSpatialReference( poSRS );
-                    poFeature->SetGeometryDirectly( poGeometry );
+                    poGeometry->assignSpatialReference( poGeomFieldDefn->GetSpatialRef() );
+                    poFeature->SetGeomFieldDirectly(iOGRGeomField,  poGeometry );
                 }
 
                 continue;
@@ -804,7 +794,8 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
 /*      Handle raw binary geometry ... this hasn't been tested in a     */
 /*      while.                                                          */
 /* -------------------------------------------------------------------- */
-        else if( EQUAL(pszFieldName,"WKB_GEOMETRY") )
+        else if( iOGRGeomField >= 0 && 
+                 poGeomFieldDefn->ePostgisType == GEOM_TYPE_WKB )
         {
             OGRGeometry *poGeometry = NULL;
             GByte* pabyData = (GByte*) PQgetvalue( hCursorResult, iRecord, iField);
@@ -823,7 +814,7 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
                    )
                 {
                     int nLength = PQgetlength(hCursorResult, iRecord, iField);
-                    poGeometry = EWKBToGeometry(pabyData, nLength);
+                    poGeometry = OGRGeometryFromEWKB(pabyData, nLength, NULL);
                 }
                 if (poGeometry == NULL)
                 {
@@ -833,8 +824,8 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
 
             if( poGeometry != NULL )
             {
-                poGeometry->assignSpatialReference( poSRS );
-                poFeature->SetGeometryDirectly( poGeometry );
+                poGeometry->assignSpatialReference( poGeomFieldDefn->GetSpatialRef() );
+                poFeature->SetGeomFieldDirectly(iOGRGeomField,  poGeometry );
             }
 
             continue;
@@ -1277,6 +1268,24 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
     return poFeature;
 }
 
+/************************************************************************/
+/*                    OGRPGIsKnownGeomFuncPrefix()                      */
+/************************************************************************/
+
+static const char* papszKnownGeomFuncPrefixes[] = {
+    "ST_AsBinary", "BinaryBase64", "ST_AsEWKT", "ST_AsEWKB", "EWKBBase64",
+    "ST_AsText", "AsBinary", "asEWKT", "asEWKB", "asText" };
+static int OGRPGIsKnownGeomFuncPrefix(const char* pszFieldName)
+{
+    for(size_t i=0; i<sizeof(papszKnownGeomFuncPrefixes) / sizeof(char*); i++)
+    {
+        if( EQUALN(pszFieldName, papszKnownGeomFuncPrefixes[i],
+                   strlen(papszKnownGeomFuncPrefixes[i])) )
+            return i;
+    }
+    return -1;
+}
+
 /************************************************************************/
 /*                CreateMapFromFieldNameToIndex()                       */
 /************************************************************************/
@@ -1289,16 +1298,39 @@ void OGRPGLayer::CreateMapFromFieldNameToIndex()
 {
     CPLFree(panMapFieldNameToIndex);
     panMapFieldNameToIndex = NULL;
+    CPLFree(panMapFieldNameToGeomIndex);
+    panMapFieldNameToGeomIndex = NULL;
     if ( PQresultStatus(hCursorResult)  == PGRES_TUPLES_OK )
     {
         panMapFieldNameToIndex =
                 (int*)CPLMalloc(sizeof(int) * PQnfields(hCursorResult));
+        panMapFieldNameToGeomIndex =
+                (int*)CPLMalloc(sizeof(int) * PQnfields(hCursorResult));
         for( int iField = 0;
             iField < PQnfields(hCursorResult);
             iField++ )
         {
+            const char* pszName = PQfname(hCursorResult,iField);
             panMapFieldNameToIndex[iField] =
-                    poFeatureDefn->GetFieldIndex(PQfname(hCursorResult,iField));
+                    poFeatureDefn->GetFieldIndex(pszName);
+            if( panMapFieldNameToIndex[iField] < 0 )
+            {
+                panMapFieldNameToGeomIndex[iField] =
+                        poFeatureDefn->GetGeomFieldIndex(pszName);
+                if( panMapFieldNameToGeomIndex[iField] < 0 )
+                {
+                    int iKnownPrefix = OGRPGIsKnownGeomFuncPrefix(pszName);
+                    if( iKnownPrefix >= 0 &&
+                        pszName[ strlen(papszKnownGeomFuncPrefixes[iKnownPrefix]) ] == '_' )
+                    {
+                        panMapFieldNameToGeomIndex[iField] =
+                            poFeatureDefn->GetGeomFieldIndex(pszName +
+                            strlen(papszKnownGeomFuncPrefixes[iKnownPrefix]) + 1);
+                    }
+                }
+            }
+            else
+                panMapFieldNameToGeomIndex[iField] = -1;
         }
     }
 }
@@ -1476,166 +1508,6 @@ OGRErr OGRPGLayer::SetNextByIndex( long nIndex )
     return OGRERR_NONE;
 }
 
-/************************************************************************/
-/*                           HEXToGeometry()                            */
-/************************************************************************/
-
-OGRGeometry *OGRPGLayer::HEXToGeometry( const char *pszBytea )
-
-{
-    GByte   *pabyWKB;
-    int     nWKBLength=0;
-    OGRGeometry *poGeometry;
-
-    if( pszBytea == NULL )
-        return NULL;
-
-    pabyWKB = CPLHexToBinary(pszBytea, &nWKBLength);
-
-    poGeometry = EWKBToGeometry(pabyWKB, nWKBLength);
-
-    CPLFree(pabyWKB);
-
-    return poGeometry;
-}
-
-
-/************************************************************************/
-/*                          EWKBToGeometry()                            */
-/************************************************************************/
-
-OGRGeometry *OGRPGLayer::EWKBToGeometry( GByte *pabyWKB, int nLength )
-
-{
-    OGRGeometry *poGeometry = NULL;
-    unsigned int ewkbFlags = 0;
-    
-    if (nLength < 5)
-    {
-        CPLError( CE_Failure, CPLE_AppDefined,
-                  "Invalid EWKB content : %d bytes", nLength );
-        return NULL;
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Detect XYZM variant of PostGIS EWKB                             */
-/*                                                                      */
-/*      OGR does not support parsing M coordinate,                      */
-/*      so we return NULL geometry.                                     */
-/* -------------------------------------------------------------------- */
-    memcpy(&ewkbFlags, pabyWKB+1, 4);
-    OGRwkbByteOrder eByteOrder = (pabyWKB[0] == 0 ? wkbXDR : wkbNDR);
-    if( OGR_SWAP( eByteOrder ) )
-        ewkbFlags= CPL_SWAP32(ewkbFlags);
-
-    if (ewkbFlags & 0x40000000)
-    {
-        CPLError( CE_Failure, CPLE_AppDefined,
-            "Reading EWKB with 4-dimensional coordinates (XYZM) is not supported" );
-
-        return NULL;
-    }
-
-/* -------------------------------------------------------------------- */
-/*      PostGIS EWKB format includes an  SRID, but this won't be        */
-/*      understood by OGR, so if the SRID flag is set, we remove the    */
-/*      SRID (bytes at offset 5 to 8).                                  */
-/* -------------------------------------------------------------------- */
-    if( nLength > 9 &&
-        ((pabyWKB[0] == 0 /* big endian */ && (pabyWKB[1] & 0x20) )
-        || (pabyWKB[0] != 0 /* little endian */ && (pabyWKB[4] & 0x20))) )
-    {
-        memmove( pabyWKB+5, pabyWKB+9, nLength-9 );
-        nLength -= 4;
-        if( pabyWKB[0] == 0 )
-            pabyWKB[1] &= (~0x20);
-        else
-            pabyWKB[4] &= (~0x20);
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Try to ingest the geometry.                                     */
-/* -------------------------------------------------------------------- */
-    OGRGeometryFactory::createFromWkb( pabyWKB, NULL, &poGeometry, nLength );
-
-    return poGeometry;
-}
-
-/************************************************************************/
-/*                           GeometryToHex()                            */
-/************************************************************************/
-char *OGRPGLayer::GeometryToHex( OGRGeometry * poGeometry, int nSRSId )
-{
-    GByte       *pabyWKB;
-    char        *pszTextBuf;
-    char        *pszTextBufCurrent;
-    char        *pszHex;
-
-    int nWkbSize = poGeometry->WkbSize();
-    pabyWKB = (GByte *) CPLMalloc(nWkbSize);
-
-    if( poGeometry->exportToWkb( wkbNDR, pabyWKB ) != OGRERR_NONE )
-    {
-        CPLFree( pabyWKB );
-        return CPLStrdup("");
-    }
-
-    /* When converting to hex, each byte takes 2 hex characters.  In addition
-       we add in 8 characters to represent the SRID integer in hex, and
-       one for a null terminator */
-
-    int pszSize = nWkbSize*2 + 8 + 1;
-    pszTextBuf = (char *) CPLMalloc(pszSize);
-    pszTextBufCurrent = pszTextBuf;
-
-    /* Convert the 1st byte, which is the endianess flag, to hex. */
-    pszHex = CPLBinaryToHex( 1, pabyWKB );
-    strcpy(pszTextBufCurrent, pszHex );
-    CPLFree ( pszHex );
-    pszTextBufCurrent += 2;
-
-    /* Next, get the geom type which is bytes 2 through 5 */
-    GUInt32 geomType;
-    memcpy( &geomType, pabyWKB+1, 4 );
-
-    /* Now add the SRID flag if an SRID is provided */
-    if (nSRSId > 0)
-    {
-        /* Change the flag to wkbNDR (little) endianess */
-        GUInt32 nGSrsFlag = CPL_LSBWORD32( WKBSRIDFLAG );
-        /* Apply the flag */
-        geomType = geomType | nGSrsFlag;
-    }
-
-    /* Now write the geom type which is 4 bytes */
-    pszHex = CPLBinaryToHex( 4, (GByte*) &geomType );
-    strcpy(pszTextBufCurrent, pszHex );
-    CPLFree ( pszHex );
-    pszTextBufCurrent += 8;
-
-    /* Now include SRID if provided */
-    if (nSRSId > 0)
-    {
-        /* Force the srsid to wkbNDR (little) endianess */
-        GUInt32 nGSRSId = CPL_LSBWORD32( nSRSId );
-        pszHex = CPLBinaryToHex( sizeof(nGSRSId),(GByte*) &nGSRSId );
-        strcpy(pszTextBufCurrent, pszHex );
-        CPLFree ( pszHex );
-        pszTextBufCurrent += 8;
-    }
-
-    /* Copy the rest of the data over - subtract
-       5 since we already copied 5 bytes above */
-    pszHex = CPLBinaryToHex( nWkbSize - 5, pabyWKB + 5 );
-    strcpy(pszTextBufCurrent, pszHex );
-    CPLFree ( pszHex );
-
-    CPLFree( pabyWKB );
-
-    return pszTextBuf;
-}
-
-
 /************************************************************************/
 /*                        BYTEAToGByteArray()                           */
 /************************************************************************/
@@ -1867,37 +1739,6 @@ OGRErr OGRPGLayer::RollbackTransaction()
     return poDS->SoftRollback();
 }
 
-/************************************************************************/
-/*                           GetSpatialRef()                            */
-/************************************************************************/
-
-OGRSpatialReference *OGRPGLayer::GetSpatialRef()
-
-{
-    if (nSRSId == UNDETERMINED_SRID)
-        GetLayerDefn();
-
-    if( poSRS == NULL && nSRSId > 0 )
-    {
-        poSRS = poDS->FetchSRS( nSRSId );
-        if( poSRS != NULL )
-            poSRS->Reference();
-    }
-
-    return poSRS;
-}
-
-/************************************************************************/
-/*                             GetSRID()                                */
-/************************************************************************/
-
-int OGRPGLayer::GetSRID()
-{
-    if (nSRSId == UNDETERMINED_SRID)
-        GetSpatialRef();
-    return nSRSId;
-};
-
 /************************************************************************/
 /*                            GetFIDColumn()                            */
 /************************************************************************/
@@ -1914,18 +1755,62 @@ const char *OGRPGLayer::GetFIDColumn()
 }
 
 /************************************************************************/
-/*                         GetGeometryColumn()                          */
+/*                             GetExtent()                              */
+/*                                                                      */
+/*      For PostGIS use internal Extend(geometry) function              */
+/*      in other cases we use standard OGRLayer::GetExtent()            */
 /************************************************************************/
 
-const char *OGRPGLayer::GetGeometryColumn() 
-
+OGRErr OGRPGLayer::GetExtent( int iGeomField, OGREnvelope *psExtent, int bForce )
 {
-    GetLayerDefn();
+    CPLString   osCommand;
 
-    if( pszGeomColumn != NULL )
-        return pszGeomColumn;
+    if( iGeomField < 0 || iGeomField >= GetLayerDefn()->GetGeomFieldCount() ||
+        GetLayerDefn()->GetGeomFieldDefn(iGeomField)->GetType() == wkbNone )
+    {
+        if( iGeomField != 0 )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Invalid geometry field index : %d", iGeomField);
+        }
+        return OGRERR_FAILURE;
+    }
+
+    OGRPGGeomFieldDefn* poGeomFieldDefn =
+        poFeatureDefn->myGetGeomFieldDefn(iGeomField);
+
+    const char* pszExtentFct;
+    if (poDS->sPostGISVersion.nMajor >= 2)
+        pszExtentFct = "ST_Extent";
     else
-        return "";
+        pszExtentFct = "Extent";
+
+    if ( TestCapability(OLCFastGetExtent) )
+    {
+        /* Do not take the spatial filter into account */
+        osCommand.Printf( "SELECT %s(%s) FROM %s AS ogrpgextent",
+                          pszExtentFct,
+                          OGRPGEscapeColumnName(poGeomFieldDefn->GetNameRef()).c_str(),
+                          GetFromClauseForGetExtent().c_str() );
+    }
+    else if ( poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOGRAPHY )
+    {
+        /* Probably not very efficient, but more efficient than client-side implementation */
+        osCommand.Printf( "SELECT %s(ST_GeomFromWKB(ST_AsBinary(%s))) FROM %s AS ogrpgextent",
+                          pszExtentFct,
+                          OGRPGEscapeColumnName(poGeomFieldDefn->GetNameRef()).c_str(),
+                          GetFromClauseForGetExtent().c_str() );
+    }
+
+    if( osCommand.size() != 0 )
+    {
+        if( RunGetExtentRequest(psExtent, bForce, osCommand) == OGRERR_NONE )
+            return OGRERR_NONE;
+    }
+    if( iGeomField == 0 )
+        return OGRLayer::GetExtent( psExtent, bForce );
+    else
+        return OGRLayer::GetExtent( iGeomField, psExtent, bForce );
 }
 
 /************************************************************************/
@@ -1938,75 +1823,70 @@ OGRErr OGRPGLayer::RunGetExtentRequest( OGREnvelope *psExtent, int bForce,
     if ( psExtent == NULL )
         return OGRERR_FAILURE;
 
-    if ( TestCapability(OLCFastGetExtent) || bHasPostGISGeography )
+    PGconn      *hPGConn = poDS->GetPGConn();
+    PGresult    *hResult = NULL;
+
+    hResult = OGRPG_PQexec( hPGConn, osCommand );
+    if( ! hResult || PQresultStatus(hResult) != PGRES_TUPLES_OK || PQgetisnull(hResult,0,0) )
     {
-        PGconn      *hPGConn = poDS->GetPGConn();
-        PGresult    *hResult = NULL;
-
-        hResult = OGRPG_PQexec( hPGConn, osCommand );
-        if( ! hResult || PQresultStatus(hResult) != PGRES_TUPLES_OK || PQgetisnull(hResult,0,0) )
-        {
-            OGRPGClearResult( hResult );
-            CPLDebug("PG","Unable to get extent by PostGIS. Using standard OGRLayer method.");
-            return OGRPGLayer::GetExtent( psExtent, bForce );
-        }
-
-        char * pszBox = PQgetvalue(hResult,0,0);
-        char * ptr, *ptrEndParenthesis;
-        char szVals[64*6+6];
-
-        ptr = strchr(pszBox, '(');
-        if (ptr)
-            ptr ++;
-        if (ptr == NULL ||
-            (ptrEndParenthesis = strchr(ptr, ')')) == NULL ||
-            ptrEndParenthesis - ptr > (int)(sizeof(szVals) - 1))
-        {
-            CPLError( CE_Failure, CPLE_IllegalArg,
-                      "Bad extent representation: '%s'", pszBox);
-
-            OGRPGClearResult( hResult );
-            return OGRERR_FAILURE;
-        }
-
-        strncpy(szVals,ptr,ptrEndParenthesis - ptr);
-        szVals[ptrEndParenthesis - ptr] = '\0';
-
-        char ** papszTokens = CSLTokenizeString2(szVals," ,",CSLT_HONOURSTRINGS);
-        int nTokenCnt = poDS->sPostGISVersion.nMajor >= 1 ? 4 : 6;
-
-        if ( CSLCount(papszTokens) != nTokenCnt )
-        {
-            CPLError( CE_Failure, CPLE_IllegalArg,
-                      "Bad extent representation: '%s'", pszBox);
-            CSLDestroy(papszTokens);
-
-            OGRPGClearResult( hResult );
-            return OGRERR_FAILURE;
-        }
-
-        // Take X,Y coords
-        // For PostGis ver >= 1.0.0 -> Tokens: X1 Y1 X2 Y2 (nTokenCnt = 4)
-        // For PostGIS ver < 1.0.0 -> Tokens: X1 Y1 Z1 X2 Y2 Z2 (nTokenCnt = 6)
-        // =>   X2 index calculated as nTokenCnt/2
-        //      Y2 index caluclated as nTokenCnt/2+1
-        
-        psExtent->MinX = CPLAtof( papszTokens[0] );
-        psExtent->MinY = CPLAtof( papszTokens[1] );
-        psExtent->MaxX = CPLAtof( papszTokens[nTokenCnt/2] );
-        psExtent->MaxY = CPLAtof( papszTokens[nTokenCnt/2+1] );
-
-        CSLDestroy(papszTokens);
         OGRPGClearResult( hResult );
-
-        return OGRERR_NONE;
+        CPLDebug("PG","Unable to get extent by PostGIS.");
+        return OGRERR_FAILURE;
     }
 
-    return OGRLayer::GetExtent( psExtent, bForce );
+    char * pszBox = PQgetvalue(hResult,0,0);
+    char * ptr, *ptrEndParenthesis;
+    char szVals[64*6+6];
+
+    ptr = strchr(pszBox, '(');
+    if (ptr)
+        ptr ++;
+    if (ptr == NULL ||
+        (ptrEndParenthesis = strchr(ptr, ')')) == NULL ||
+        ptrEndParenthesis - ptr > (int)(sizeof(szVals) - 1))
+    {
+        CPLError( CE_Failure, CPLE_IllegalArg,
+                    "Bad extent representation: '%s'", pszBox);
+
+        OGRPGClearResult( hResult );
+        return OGRERR_FAILURE;
+    }
+
+    strncpy(szVals,ptr,ptrEndParenthesis - ptr);
+    szVals[ptrEndParenthesis - ptr] = '\0';
+
+    char ** papszTokens = CSLTokenizeString2(szVals," ,",CSLT_HONOURSTRINGS);
+    int nTokenCnt = poDS->sPostGISVersion.nMajor >= 1 ? 4 : 6;
+
+    if ( CSLCount(papszTokens) != nTokenCnt )
+    {
+        CPLError( CE_Failure, CPLE_IllegalArg,
+                    "Bad extent representation: '%s'", pszBox);
+        CSLDestroy(papszTokens);
+
+        OGRPGClearResult( hResult );
+        return OGRERR_FAILURE;
+    }
+
+    // Take X,Y coords
+    // For PostGis ver >= 1.0.0 -> Tokens: X1 Y1 X2 Y2 (nTokenCnt = 4)
+    // For PostGIS ver < 1.0.0 -> Tokens: X1 Y1 Z1 X2 Y2 Z2 (nTokenCnt = 6)
+    // =>   X2 index calculated as nTokenCnt/2
+    //      Y2 index caluclated as nTokenCnt/2+1
+    
+    psExtent->MinX = CPLAtof( papszTokens[0] );
+    psExtent->MinY = CPLAtof( papszTokens[1] );
+    psExtent->MaxX = CPLAtof( papszTokens[nTokenCnt/2] );
+    psExtent->MaxY = CPLAtof( papszTokens[nTokenCnt/2+1] );
+
+    CSLDestroy(papszTokens);
+    OGRPGClearResult( hResult );
+
+    return OGRERR_NONE;
 }
 
 /************************************************************************/
-/*                         GetLayerDefn()                              */
+/*                          GetLayerDefn()                              */
 /************************************************************************/
 
 OGRFeatureDefn * OGRPGLayer::GetLayerDefn()
@@ -2020,7 +1900,7 @@ OGRFeatureDefn * OGRPGLayer::GetLayerDefn()
 /*      Build a schema from the current resultset.                      */
 /************************************************************************/
 
-OGRFeatureDefn *OGRPGLayer::ReadResultDefinition(PGresult *hInitialResultIn)
+int OGRPGLayer::ReadResultDefinition(PGresult *hInitialResultIn)
 
 {
     PGresult            *hResult = hInitialResultIn;
@@ -2028,10 +1908,10 @@ OGRFeatureDefn *OGRPGLayer::ReadResultDefinition(PGresult *hInitialResultIn)
 /* -------------------------------------------------------------------- */
 /*      Parse the returned table information.                           */
 /* -------------------------------------------------------------------- */
-    OGRFeatureDefn *poDefn = new OGRFeatureDefn( "sql_statement" );
+    poFeatureDefn = new OGRPGFeatureDefn( "sql_statement" );
     int            iRawField;
 
-    poDefn->Reference();
+    poFeatureDefn->Reference();
 
     for( iRawField = 0; iRawField < PQnfields(hResult); iRawField++ )
     {
@@ -2040,6 +1920,7 @@ OGRFeatureDefn *OGRPGLayer::ReadResultDefinition(PGresult *hInitialResultIn)
 
         nTypeOID = PQftype(hResult,iRawField);
 
+        int iGeomFuncPrefix;
         if( EQUAL(oField.GetNameRef(),"ogc_fid") )
         {
             if (bHasFid)
@@ -2052,37 +1933,38 @@ OGRFeatureDefn *OGRPGLayer::ReadResultDefinition(PGresult *hInitialResultIn)
             pszFIDColumn = CPLStrdup(oField.GetNameRef());
             continue;
         }
-        else if( nTypeOID == poDS->GetGeometryOID()  ||
-                 nTypeOID == poDS->GetGeographyOID()  ||
-                 EQUAL(oField.GetNameRef(),"ST_AsBinary") ||
-                 EQUAL(oField.GetNameRef(),"BinaryBase64") ||
-                 EQUAL(oField.GetNameRef(),"ST_AsEWKT") ||
-                 EQUAL(oField.GetNameRef(),"ST_AsEWKB") ||
-                 EQUAL(oField.GetNameRef(),"EWKBBase64") ||
-                 EQUAL(oField.GetNameRef(),"ST_AsText") ||
-                 EQUAL(oField.GetNameRef(),"AsBinary") ||
-                 EQUAL(oField.GetNameRef(),"asEWKT") ||
-                 EQUAL(oField.GetNameRef(),"asEWKB") ||
-                 EQUAL(oField.GetNameRef(),"asText") )
+        else if( (iGeomFuncPrefix =
+                    OGRPGIsKnownGeomFuncPrefix(oField.GetNameRef())) >= 0 ||
+                 nTypeOID == poDS->GetGeometryOID()  ||
+                 nTypeOID == poDS->GetGeographyOID()  )
         {
-            if (bHasPostGISGeometry || bHasPostGISGeography )
+            OGRPGGeomFieldDefn* poGeomFieldDefn =
+                new OGRPGGeomFieldDefn(this, oField.GetNameRef());
+            if( iGeomFuncPrefix >= 0 &&
+                oField.GetNameRef()[strlen(
+                    papszKnownGeomFuncPrefixes[iGeomFuncPrefix])] == '_' )
             {
-                CPLError(CE_Warning, CPLE_AppDefined,
-                         "More than one geometry column was found in the result of the SQL request. Only last one will be used");
+                poGeomFieldDefn->SetName( oField.GetNameRef() +
+                    strlen(papszKnownGeomFuncPrefixes[iGeomFuncPrefix]) + 1 );
             }
             if (nTypeOID == poDS->GetGeographyOID())
-                bHasPostGISGeography = TRUE;
+            {
+                poGeomFieldDefn->ePostgisType = GEOM_TYPE_GEOGRAPHY;
+                poGeomFieldDefn->nSRSId = 4326;
+            }
             else
-                bHasPostGISGeometry = TRUE;
-            CPLFree(pszGeomColumn);
-            pszGeomColumn = CPLStrdup(oField.GetNameRef());
+                poGeomFieldDefn->ePostgisType = GEOM_TYPE_GEOMETRY;
+            poFeatureDefn->AddGeomFieldDefn(poGeomFieldDefn, FALSE);
             continue;
         }
         else if( EQUAL(oField.GetNameRef(),"WKB_GEOMETRY") )
         {
-            bHasWkb = TRUE;
             if( nTypeOID == OIDOID )
                 bWkbAsOid = TRUE;
+            OGRPGGeomFieldDefn* poGeomFieldDefn =
+                new OGRPGGeomFieldDefn(this, oField.GetNameRef());
+            poGeomFieldDefn->ePostgisType = GEOM_TYPE_WKB;
+            poFeatureDefn->AddGeomFieldDefn(poGeomFieldDefn, FALSE);
             continue;
         }
 
@@ -2193,8 +2075,28 @@ OGRFeatureDefn *OGRPGLayer::ReadResultDefinition(PGresult *hInitialResultIn)
             oField.SetType( OFTString );
         }
 
-        poDefn->AddFieldDefn( &oField );
+        poFeatureDefn->AddFieldDefn( &oField );
     }
 
-    return poDefn;
+    return TRUE;
+}
+
+/************************************************************************/
+/*                          GetSpatialRef()                             */
+/************************************************************************/
+
+OGRSpatialReference* OGRPGGeomFieldDefn::GetSpatialRef()
+{
+    if( poLayer == NULL )
+        return NULL;
+    if (nSRSId == UNDETERMINED_SRID)
+        poLayer->ResolveSRID(this);
+
+    if( poSRS == NULL && nSRSId > 0 )
+    {
+        poSRS = poLayer->GetDS()->FetchSRS( nSRSId );
+        if( poSRS != NULL )
+            poSRS->Reference();
+    }
+    return poSRS;
 }
