@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpgeodatasource.cpp 10645 2007-01-18 02:22:39Z warmerdam $
+ * $Id: ogrpgeodatasource.cpp 27044 2014-03-16 23:41:27Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGeoDataSource class.
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
+ * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +33,7 @@
 #include "cpl_string.h"
 #include <vector>
 
-CPL_CVSID("$Id: ogrpgeodatasource.cpp 10645 2007-01-18 02:22:39Z warmerdam $");
+CPL_CVSID("$Id: ogrpgeodatasource.cpp 27044 2014-03-16 23:41:27Z rouault $");
 
 /************************************************************************/
 /*                         OGRPGeoDataSource()                          */
@@ -63,6 +64,36 @@ OGRPGeoDataSource::~OGRPGeoDataSource()
     CPLFree( papoLayers );
 }
 
+/************************************************************************/
+/*                  CheckDSNStringTemplate()                            */
+/* The string will be used as the formatting argument of sprintf with   */
+/* a string in vararg. So let's check there's only one '%s', and nothing*/
+/* else                                                                 */
+/************************************************************************/
+
+static int CheckDSNStringTemplate(const char* pszStr)
+{
+    int nPercentSFound = FALSE;
+    while(*pszStr)
+    {
+        if (*pszStr == '%')
+        {
+            if (pszStr[1] != 's')
+            {
+                return FALSE;
+            }
+            else
+            {
+                if (nPercentSFound)
+                    return FALSE;
+                nPercentSFound = TRUE;
+            }
+        }
+        pszStr ++;
+    }
+    return TRUE;
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -84,9 +115,16 @@ int OGRPGeoDataSource::Open( const char * pszNewName, int bUpdate,
         pszDSN = CPLStrdup( pszNewName + 5 );
     else
     {
-        pszDSN = (char *) CPLMalloc(strlen(pszNewName)+50);
-        sprintf( pszDSN, "DRIVER=Microsoft Access Driver (*.mdb);DBQ=%s", 
-                 pszNewName );
+        const char *pszDSNStringTemplate = NULL;
+        pszDSNStringTemplate = CPLGetConfigOption( "PGEO_DRIVER_TEMPLATE", "DRIVER=Microsoft Access Driver (*.mdb);DBQ=%s");
+        if (!CheckDSNStringTemplate(pszDSNStringTemplate))
+        {
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Illegal value for PGEO_DRIVER_TEMPLATE option");
+            return FALSE;
+        }
+        pszDSN = (char *) CPLMalloc(strlen(pszNewName)+strlen(pszDSNStringTemplate)+100);
+        sprintf( pszDSN, pszDSNStringTemplate,  pszNewName );
     }
 
 /* -------------------------------------------------------------------- */
@@ -208,9 +246,9 @@ OGRLayer * OGRPGeoDataSource::ExecuteSQL( const char *pszSQLCommand,
 
 {
 /* -------------------------------------------------------------------- */
-/*      Use generic imlplementation for OGRSQL dialect.                 */
+/*      Use generic implementation for recognized dialects              */
 /* -------------------------------------------------------------------- */
-    if( pszDialect != NULL && EQUAL(pszDialect,"OGRSQL") )
+    if( IsGenericSQLDialect(pszDialect) )
         return OGRDataSource::ExecuteSQL( pszSQLCommand, 
                                           poSpatialFilter, 
                                           pszDialect );
