@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shape2ogr.cpp 26116 2013-06-29 18:57:36Z rouault $
+ * $Id: shape2ogr.cpp 27604 2014-08-23 10:01:50Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements translation of Shapefile shapes into OGR
@@ -8,6 +8,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1999,  Les Technologies SoftMap Inc.
+ * Copyright (c) 2007-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,7 +32,7 @@
 #include "ogrshape.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: shape2ogr.cpp 26116 2013-06-29 18:57:36Z rouault $");
+CPL_CVSID("$Id: shape2ogr.cpp 27604 2014-08-23 10:01:50Z rouault $");
 
 /************************************************************************/
 /*                        RingStartEnd                                  */
@@ -1318,14 +1319,38 @@ OGRErr SHPWriteOGRFeature( SHPHandle hSHP, DBFHandle hDBF,
               }
 
               DBFWriteAttributeDirectly( hDBF, poFeature->GetFID(), iField, 
-                                                szValue );
-            break;
+                                         szValue );
+
+              break;
           }
 
           case OFTReal:
-            DBFWriteDoubleAttribute( hDBF, poFeature->GetFID(), iField, 
-                                     poFeature->GetFieldAsDouble(iField) );
+          {
+            double dfVal = poFeature->GetFieldAsDouble(iField);
+            /* IEEE754 doubles can store exact values of all integers below 2^53 */
+            if( poFieldDefn->GetPrecision() == 0 && fabs(dfVal) > ((GIntBig)1 << 53) )
+            {
+                static int nCounter = 0;
+                if( nCounter <= 10 )
+                {
+                    CPLError(CE_Warning, CPLE_AppDefined,
+                             "Value %.18g of field %s with 0 decimal of feature %ld is bigger than 2^53. Precision loss likely occured or going to happen.%s",
+                             dfVal, poFieldDefn->GetNameRef(), poFeature->GetFID(),
+                             (nCounter == 10) ? " This warning will not be emitted anymore." : "");
+                    nCounter ++;
+                }
+            }
+            int ret = DBFWriteDoubleAttribute( hDBF, poFeature->GetFID(), iField, 
+                                               dfVal );
+            if( !ret )
+            {
+                CPLError(CE_Warning, CPLE_AppDefined,
+                         "Value %.18g of field %s of feature %ld not successfully written. "
+                         "Possibly due to too larger number with respect to field width",
+                         dfVal, poFieldDefn->GetNameRef(), poFeature->GetFID());
+            }
             break;
+          }
 
           case OFTDate:
           {
