@@ -1,12 +1,13 @@
 /******************************************************************************
- * $Id: vfkdatablocksqlite.cpp 25721 2013-03-09 16:21:46Z martinl $
+ * $Id: vfkdatablocksqlite.cpp 27044 2014-03-16 23:41:27Z rouault $
  *
  * Project:  VFK Reader - Data block definition (SQLite)
  * Purpose:  Implements VFKDataBlockSQLite
  * Author:   Martin Landa, landa.martin gmail.com
  *
  ******************************************************************************
- * Copyright (c) 2012-2013, Martin Landa <landa.martin gmail.com>
+ * Copyright (c) 2012-2014, Martin Landa <landa.martin gmail.com>
+ * Copyright (c) 2012-2014, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -92,7 +93,7 @@ int VFKDataBlockSQLite::LoadGeometryPoint()
 	    nGeometries++;
     }
     
-    /* update number of geometries in 'vfk_blocks' table */
+    /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
 
     if (poReader->IsSpatial())
@@ -211,13 +212,13 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
 		}
 		else {
 		    CPLDebug("OGR-VFK", 
-			     "Geometry (point ID = %lld) not valid", id);
+			     "Geometry (point ID = " CPL_FRMT_GUIB ") not valid", id);
 		    bValid = FALSE;
 		}
 	    }
 	    else {
                 CPLDebug("OGR-VFK", 
-                         "Point ID = %lld not found (rowid = %d)",
+                         "Point ID = " CPL_FRMT_GUIB " not found (rowid = %d)",
                          id, rowId);
 		bValid = FALSE;
             }
@@ -256,7 +257,7 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
 	    poReader->ExecuteSQL("COMMIT");
     }
 
-    /* update number of geometries in 'vfk_blocks' table */
+    /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
     
     return nInvalid;
@@ -330,6 +331,9 @@ int VFKDataBlockSQLite::LoadGeometryLineStringHP()
 	    poOgrGeometry = poLine->GetGeometry();
 	}
 	if (!poOgrGeometry || !poFeature->SetGeometry(poOgrGeometry)) {
+            CPLDebug("OGR-VFK", "VFKDataBlockSQLite::LoadGeometryLineStringHP(): name=%s fid=%ld "
+                     "id=" CPL_FRMT_GUIB " -> %s geometry", m_pszName, iFID, vrValue[0],
+                     poOgrGeometry ? "invalid" : "empty");
 	    nInvalid++;
             continue;
         }
@@ -341,7 +345,7 @@ int VFKDataBlockSQLite::LoadGeometryLineStringHP()
 	    nGeometries++;
     }
     
-    /* update number of geometries in 'vfk_blocks' table */
+    /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
     
     if (poReader->IsSpatial())
@@ -597,7 +601,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
     CPLDebug("OGR-VFK", "%s: nolines = %d norings = %d",
              m_pszName, nInvalidNoLines, nInvalidNoRings);
     
-    /* update number of geometries in 'vfk_blocks' table */
+    /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
 
     if (poReader->IsSpatial())
@@ -686,10 +690,10 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char *column, GUIntBig va
         return NULL;
     
     idx = sqlite3_column_int(hStmt, 0) - 1;
+    sqlite3_finalize(hStmt);
+    
     if (idx < 0 || idx >= m_nFeatureCount) // ? assert
         return NULL;
-    
-    sqlite3_finalize(hStmt);
 
     return (VFKFeatureSQLite *) GetFeatureByIndex(idx);
 }
@@ -733,11 +737,10 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char **column, GUIntBig *
         return NULL;
     
     idx = sqlite3_column_int(hStmt, 0) - 1; /* rowid starts at 1 */
-    
+    sqlite3_finalize(hStmt);    
+
     if (idx < 0 || idx >= m_nFeatureCount) // ? assert
         return NULL;
-
-    sqlite3_finalize(hStmt);
     
     return (VFKFeatureSQLite *) GetFeatureByIndex(idx);
 }
@@ -854,8 +857,8 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
     if (!poReader->IsSpatial())   /* check if DB is spatial */
 	return FALSE;
 
-    osSQL.Printf("SELECT num_geometries FROM 'vfk_blocks' WHERE table_name = '%s'",
-		 m_pszName);
+    osSQL.Printf("SELECT num_geometries FROM %s WHERE table_name = '%s'",
+		 VFK_DB_TABLE, m_pszName);
     hStmt = poReader->PrepareStatement(osSQL.c_str());
     if (poReader->ExecuteSQL(hStmt) != OGRERR_NONE)
         return FALSE;
@@ -901,6 +904,7 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
 	    if (!poFeature->SetGeometry(poGeometry)) {
 		nInvalid++;
 	    }
+	    delete poGeometry;
 	}
 	else {
 	    nInvalid++;
@@ -918,7 +922,7 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
 
     if (nInvalid > 0 && !bSkipInvalid) {
 	CPLError(CE_Warning, CPLE_AppDefined, 
-                 "%s: %d invalid features found",
+                 "%s: %d features with invalid or empty geometry found",
 		 m_pszName, nInvalid);
     }
 
@@ -926,7 +930,7 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
 }
 
 /*!
-  \brief Update vfk_blocks table
+  \brief Update VFK_DB_TABLE table
 
   \param nGeometries number of geometries to update
 */
@@ -939,11 +943,12 @@ void VFKDataBlockSQLite::UpdateVfkBlocks(int nGeometries) {
 
     if (nGeometries > 0) {
         CPLDebug("OGR-VFK", 
-                 "%d geometries in '%s' saved", nGeometries, m_pszName);
+                 "VFKDataBlockSQLite::UpdateVfkBlocks(): name=%s -> "
+                 "%d geometries saved to internal DB", m_pszName, nGeometries);
         
-	/* update number of geometries in 'vfk_blocks' table */
-	osSQL.Printf("UPDATE vfk_blocks SET num_geometries = %d WHERE table_name = '%s'",
-		     nGeometries, m_pszName);
+	/* update number of geometries in VFK_DB_TABLE table */
+	osSQL.Printf("UPDATE %s SET num_geometries = %d WHERE table_name = '%s'",
+		     VFK_DB_TABLE, nGeometries, m_pszName);
 	poReader->ExecuteSQL(osSQL.c_str());
     }
 }
@@ -961,7 +966,7 @@ void VFKDataBlockSQLite::UpdateFID(long int iFID, std::vector<int> rowId)
     
     poReader = (VFKReaderSQLite*) m_poReader;
     
-    /* update number of geometries in 'vfk_blocks' table */
+    /* update number of geometries in VFK_DB_TABLE table */
     osSQL.Printf("UPDATE %s SET %s = %ld WHERE rowid IN (",
                  m_pszName, FID_COLUMN, iFID);
     for (size_t i = 0; i < rowId.size(); i++) {
